from __future__ import print_function
from __future__ import division
'''
# Copyright (C) 2015, Elphel.inc.
# Module to keep globals (Verilog parameters) accessible for all modules
# that import (not import from) this one
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

@author:     Andrey Filippov
@copyright:  2015 Elphel, Inc.
@license:    GPLv3.0+
@contact:    andrey@elphel.coml
@deffield    updated: Updated
'''
__author__ = "Andrey Filippov"
__copyright__ = "Copyright 2015, Elphel, Inc."
__license__ = "GPL"
__version__ = "3.0+"
__maintainer__ = "Andrey Filippov"
__email__ = "andrey@elphel.com"
__status__ = "Development"
DEFAULTS={}
dqs_dqm_patt=None
def init_vars(d):
    global DEFAULTS
    if d:
        globals().update(d)
        for k,v in d.items():
            DEFAULTS[k]=v
def set_name_field(vname,
                   fieldIndex,
                   value):
    """
    Set specified byte in the parameter
    <vname>      Verilog parameter name string (as listen in 'parameters')
    <fieldIndex> byte field index (0 - lowest byte, 1 - bits[15:8], etc)
    <value>      value to set the specified byte to  
    """
    v=globals()[vname]
    mask = 0xff << (8*fieldIndex)
    val = value << (8*fieldIndex)
    v = ((v ^ val) & mask) ^ v
    globals()[vname]=v

def get_name_field(vname,
                   fieldIndex):
    """
    Get specified byte in the parameter
    <vname>      Verilog parameter name string (as listen in 'parameters')
    <fieldIndex> byte field index (0 - lowest byte, 1 - bits[15:8], etc)
    Return  specified byte
    """
#    print ("value for %s is %s"%(vname,str(globals()[vname])))
#    print ("dflt  for %s is %s"%(vname,str(DEFAULTS[vname])))
#    print ("value[%d] for %s is 0x%x"%(fieldIndex,vname,globals()[vname]))
#    print ("dflt[%d]  for %s is 0x%x"%(fieldIndex,vname,DEFAULTS[vname]))
    
    return ( globals()[vname] >> (8*fieldIndex)) & 0xff

def get_default_field(vname,
                      fieldIndex):
    """
    Get specified byte in the parameter default value (read at program
    start from Verilog parameters)
    <vname>      Verilog parameter name string (as listen in 'parameters')
    <fieldIndex> byte field index (0 - lowest byte, 1 - bits[15:8], etc)
    Return  specified byte
    """
    global DEFAULTS
#    print ("value for %s is %s"%(vname,str(globals()[vname])))
#    print ("dflt  for %s is %s"%(vname,str(DEFAULTS[vname])))
#    print ("value[%d] for %s is 0x%x"%(fieldIndex,vname,globals()[vname]))
#    print ("dflt[%d]  for %s is 0x%x"%(fieldIndex,vname,DEFAULTS[vname]))

    return (DEFAULTS[vname] >> (8*fieldIndex)) & 0xff

def get_default(vname):
    """
    Get parameter default value (read at program start from Verilog parameters)
    <vname>      Verilog parameter name string (as listen in 'parameters')
    Return       specified parameter default
    """
    global DEFAULTS
    return DEFAULTS[vname]
    
def save_default(vname=None):
    """
    Save parameter default value (replace read at program start from Verilog
                 parameters) using current parameter value
    <vname>      Verilog parameter name string (as listen in 'parameters')
    """
    global DEFAULTS
    if vname and vname in DEFAULTS:
        DEFAULTS[vname] =  globals()[vname]
    else:
        for vname in DEFAULTS:
            DEFAULTS[vname] =  globals()[vname]

def restore_default(vname=None):
    """
    Restore parameter value from default
    <vname>      Verilog parameter name string (as listen in 'parameters')
    """
    global DEFAULTS
    if vname:
        globals()[vname] = DEFAULTS[vname]
    else:
        for vname in DEFAULTS:
            globals()[vname] = DEFAULTS[vname]

#### PyDev predefines

PICKLE = str
PICKLE__TYPE = str
PICKLE__RAW = str
VERBOSE = int
VERBOSE__TYPE = str
VERBOSE__RAW = str
MCONTR_WR_MASK = int
MCONTR_WR_MASK__TYPE = str
MCONTR_WR_MASK__RAW = str
MCONTR_RD_MASK = int
MCONTR_RD_MASK__TYPE = str
MCONTR_RD_MASK__RAW = str
CONTROL_ADDR = int
CONTROL_ADDR__TYPE = str
CONTROL_ADDR__RAW = str
CONTROL_ADDR_MASK = int
CONTROL_ADDR_MASK__TYPE = str
CONTROL_ADDR_MASK__RAW = str
CONTROL_RBACK_ADDR = int
CONTROL_RBACK_ADDR__TYPE = str
CONTROL_RBACK_ADDR__RAW = str
CONTROL_RBACK_ADDR_MASK = int
CONTROL_RBACK_ADDR_MASK__TYPE = str
CONTROL_RBACK_ADDR_MASK__RAW = str
CONTROL_RBACK_DEPTH = int
CONTROL_RBACK_DEPTH__TYPE = str
CONTROL_RBACK_DEPTH__RAW = str
STATUS_ADDR = int
STATUS_ADDR__TYPE = str
STATUS_ADDR__RAW = str
STATUS_ADDR_MASK = int
STATUS_ADDR_MASK__TYPE = str
STATUS_ADDR_MASK__RAW = str
MCONTR_CMD_WR_ADDR = int
MCONTR_CMD_WR_ADDR__TYPE = str
MCONTR_CMD_WR_ADDR__RAW = str
MCONTR_BUF0_RD_ADDR = int
MCONTR_BUF0_RD_ADDR__TYPE = str
MCONTR_BUF0_RD_ADDR__RAW = str
MCONTR_BUF0_WR_ADDR = int
MCONTR_BUF0_WR_ADDR__TYPE = str
MCONTR_BUF0_WR_ADDR__RAW = str
MCONTR_BUF2_RD_ADDR = int
MCONTR_BUF2_RD_ADDR__TYPE = str
MCONTR_BUF2_RD_ADDR__RAW = str
MCONTR_BUF2_WR_ADDR = int
MCONTR_BUF2_WR_ADDR__TYPE = str
MCONTR_BUF2_WR_ADDR__RAW = str
MCONTR_BUF3_RD_ADDR = int
MCONTR_BUF3_RD_ADDR__TYPE = str
MCONTR_BUF3_RD_ADDR__RAW = str
MCONTR_BUF3_WR_ADDR = int
MCONTR_BUF3_WR_ADDR__TYPE = str
MCONTR_BUF3_WR_ADDR__RAW = str
MCONTR_BUF4_RD_ADDR = int
MCONTR_BUF4_RD_ADDR__TYPE = str
MCONTR_BUF4_RD_ADDR__RAW = str
MCONTR_BUF4_WR_ADDR = int
MCONTR_BUF4_WR_ADDR__TYPE = str
MCONTR_BUF4_WR_ADDR__RAW = str
AXI_WR_ADDR_BITS = int
AXI_WR_ADDR_BITS__TYPE = str
AXI_WR_ADDR_BITS__RAW = str
AXI_RD_ADDR_BITS = int
AXI_RD_ADDR_BITS__TYPE = str
AXI_RD_ADDR_BITS__RAW = str
STATUS_DEPTH = int
STATUS_DEPTH__TYPE = str
STATUS_DEPTH__RAW = str
DLY_LD = int
DLY_LD__TYPE = str
DLY_LD__RAW = str
DLY_LD_MASK = int
DLY_LD_MASK__TYPE = str
DLY_LD_MASK__RAW = str
MCONTR_PHY_0BIT_ADDR = int
MCONTR_PHY_0BIT_ADDR__TYPE = str
MCONTR_PHY_0BIT_ADDR__RAW = str
MCONTR_PHY_0BIT_ADDR_MASK = int
MCONTR_PHY_0BIT_ADDR_MASK__TYPE = str
MCONTR_PHY_0BIT_ADDR_MASK__RAW = str
MCONTR_PHY_0BIT_DLY_SET = int
MCONTR_PHY_0BIT_DLY_SET__TYPE = str
MCONTR_PHY_0BIT_DLY_SET__RAW = str
MCONTR_PHY_0BIT_CMDA_EN = int
MCONTR_PHY_0BIT_CMDA_EN__TYPE = str
MCONTR_PHY_0BIT_CMDA_EN__RAW = str
MCONTR_PHY_0BIT_SDRST_ACT = int
MCONTR_PHY_0BIT_SDRST_ACT__TYPE = str
MCONTR_PHY_0BIT_SDRST_ACT__RAW = str
MCONTR_PHY_0BIT_CKE_EN = int
MCONTR_PHY_0BIT_CKE_EN__TYPE = str
MCONTR_PHY_0BIT_CKE_EN__RAW = str
MCONTR_PHY_0BIT_DCI_RST = int
MCONTR_PHY_0BIT_DCI_RST__TYPE = str
MCONTR_PHY_0BIT_DCI_RST__RAW = str
MCONTR_PHY_0BIT_DLY_RST = int
MCONTR_PHY_0BIT_DLY_RST__TYPE = str
MCONTR_PHY_0BIT_DLY_RST__RAW = str
MCONTR_TOP_0BIT_ADDR = int
MCONTR_TOP_0BIT_ADDR__TYPE = str
MCONTR_TOP_0BIT_ADDR__RAW = str
MCONTR_TOP_0BIT_ADDR_MASK = int
MCONTR_TOP_0BIT_ADDR_MASK__TYPE = str
MCONTR_TOP_0BIT_ADDR_MASK__RAW = str
MCONTR_TOP_0BIT_MCONTR_EN = int
MCONTR_TOP_0BIT_MCONTR_EN__TYPE = str
MCONTR_TOP_0BIT_MCONTR_EN__RAW = str
MCONTR_TOP_0BIT_REFRESH_EN = int
MCONTR_TOP_0BIT_REFRESH_EN__TYPE = str
MCONTR_TOP_0BIT_REFRESH_EN__RAW = str
MCONTR_PHY_16BIT_ADDR = int
MCONTR_PHY_16BIT_ADDR__TYPE = str
MCONTR_PHY_16BIT_ADDR__RAW = str
MCONTR_PHY_16BIT_ADDR_MASK = int
MCONTR_PHY_16BIT_ADDR_MASK__TYPE = str
MCONTR_PHY_16BIT_ADDR_MASK__RAW = str
MCONTR_PHY_16BIT_PATTERNS = int
MCONTR_PHY_16BIT_PATTERNS__TYPE = str
MCONTR_PHY_16BIT_PATTERNS__RAW = str
MCONTR_PHY_16BIT_PATTERNS_TRI = int
MCONTR_PHY_16BIT_PATTERNS_TRI__TYPE = str
MCONTR_PHY_16BIT_PATTERNS_TRI__RAW = str
MCONTR_PHY_16BIT_WBUF_DELAY = int
MCONTR_PHY_16BIT_WBUF_DELAY__TYPE = str
MCONTR_PHY_16BIT_WBUF_DELAY__RAW = str
MCONTR_PHY_16BIT_EXTRA = int
MCONTR_PHY_16BIT_EXTRA__TYPE = str
MCONTR_PHY_16BIT_EXTRA__RAW = str
MCONTR_PHY_STATUS_CNTRL = int
MCONTR_PHY_STATUS_CNTRL__TYPE = str
MCONTR_PHY_STATUS_CNTRL__RAW = str
MCONTR_ARBIT_ADDR = int
MCONTR_ARBIT_ADDR__TYPE = str
MCONTR_ARBIT_ADDR__RAW = str
MCONTR_ARBIT_ADDR_MASK = int
MCONTR_ARBIT_ADDR_MASK__TYPE = str
MCONTR_ARBIT_ADDR_MASK__RAW = str
MCONTR_TOP_16BIT_ADDR = int
MCONTR_TOP_16BIT_ADDR__TYPE = str
MCONTR_TOP_16BIT_ADDR__RAW = str
MCONTR_TOP_16BIT_ADDR_MASK = int
MCONTR_TOP_16BIT_ADDR_MASK__TYPE = str
MCONTR_TOP_16BIT_ADDR_MASK__RAW = str
MCONTR_TOP_16BIT_CHN_EN = int
MCONTR_TOP_16BIT_CHN_EN__TYPE = str
MCONTR_TOP_16BIT_CHN_EN__RAW = str
MCONTR_TOP_16BIT_REFRESH_PERIOD = int
MCONTR_TOP_16BIT_REFRESH_PERIOD__TYPE = str
MCONTR_TOP_16BIT_REFRESH_PERIOD__RAW = str
MCONTR_TOP_16BIT_REFRESH_ADDRESS = int
MCONTR_TOP_16BIT_REFRESH_ADDRESS__TYPE = str
MCONTR_TOP_16BIT_REFRESH_ADDRESS__RAW = str
MCONTR_TOP_16BIT_STATUS_CNTRL = int
MCONTR_TOP_16BIT_STATUS_CNTRL__TYPE = str
MCONTR_TOP_16BIT_STATUS_CNTRL__RAW = str
MCONTR_PHY_STATUS_REG_ADDR = int
MCONTR_PHY_STATUS_REG_ADDR__TYPE = str
MCONTR_PHY_STATUS_REG_ADDR__RAW = str
MCONTR_TOP_STATUS_REG_ADDR = int
MCONTR_TOP_STATUS_REG_ADDR__TYPE = str
MCONTR_TOP_STATUS_REG_ADDR__RAW = str
CHNBUF_READ_LATENCY = int
CHNBUF_READ_LATENCY__TYPE = str
CHNBUF_READ_LATENCY__RAW = str
DFLT_DQS_PATTERN = int
DFLT_DQS_PATTERN__TYPE = str
DFLT_DQS_PATTERN__RAW = str
DFLT_DQM_PATTERN = int
DFLT_DQM_PATTERN__TYPE = str
DFLT_DQM_PATTERN__RAW = str
DFLT_DQ_TRI_ON_PATTERN = int
DFLT_DQ_TRI_ON_PATTERN__TYPE = str
DFLT_DQ_TRI_ON_PATTERN__RAW = str
DFLT_DQ_TRI_OFF_PATTERN = int
DFLT_DQ_TRI_OFF_PATTERN__TYPE = str
DFLT_DQ_TRI_OFF_PATTERN__RAW = str
DFLT_DQS_TRI_ON_PATTERN = int
DFLT_DQS_TRI_ON_PATTERN__TYPE = str
DFLT_DQS_TRI_ON_PATTERN__RAW = str
DFLT_DQS_TRI_OFF_PATTERN = int
DFLT_DQS_TRI_OFF_PATTERN__TYPE = str
DFLT_DQS_TRI_OFF_PATTERN__RAW = str
DFLT_WBUF_DELAY = int
DFLT_WBUF_DELAY__TYPE = str
DFLT_WBUF_DELAY__RAW = str
DFLT_INV_CLK_DIV = int
DFLT_INV_CLK_DIV__TYPE = str
DFLT_INV_CLK_DIV__RAW = str
DFLT_CHN_EN = int
DFLT_CHN_EN__TYPE = str
DFLT_CHN_EN__RAW = str
DFLT_REFRESH_ADDR = int
DFLT_REFRESH_ADDR__TYPE = str
DFLT_REFRESH_ADDR__RAW = str
DFLT_REFRESH_PERIOD = int
DFLT_REFRESH_PERIOD__TYPE = str
DFLT_REFRESH_PERIOD__RAW = str
ADDRESS_NUMBER = int
ADDRESS_NUMBER__TYPE = str
ADDRESS_NUMBER__RAW = str
COLADDR_NUMBER = int
COLADDR_NUMBER__TYPE = str
COLADDR_NUMBER__RAW = str
PHASE_WIDTH = int
PHASE_WIDTH__TYPE = str
PHASE_WIDTH__RAW = str
SLEW_DQ = str
SLEW_DQ__TYPE = str
SLEW_DQ__RAW = str
SLEW_DQS = str
SLEW_DQS__TYPE = str
SLEW_DQS__RAW = str
SLEW_CMDA = str
SLEW_CMDA__TYPE = str
SLEW_CMDA__RAW = str
SLEW_CLK = str
SLEW_CLK__TYPE = str
SLEW_CLK__RAW = str
IBUF_LOW_PWR = str
IBUF_LOW_PWR__TYPE = str
IBUF_LOW_PWR__RAW = str
REFCLK_FREQUENCY = float
REFCLK_FREQUENCY__TYPE = str
REFCLK_FREQUENCY__RAW = str
HIGH_PERFORMANCE_MODE = str
HIGH_PERFORMANCE_MODE__TYPE = str
HIGH_PERFORMANCE_MODE__RAW = str
CLKIN_PERIOD = int
CLKIN_PERIOD__TYPE = str
CLKIN_PERIOD__RAW = str
CLKFBOUT_MULT = int
CLKFBOUT_MULT__TYPE = str
CLKFBOUT_MULT__RAW = str
DIVCLK_DIVIDE = int
DIVCLK_DIVIDE__TYPE = str
DIVCLK_DIVIDE__RAW = str
CLKFBOUT_USE_FINE_PS = int
CLKFBOUT_USE_FINE_PS__TYPE = str
CLKFBOUT_USE_FINE_PS__RAW = str
CLKFBOUT_PHASE = float
CLKFBOUT_PHASE__TYPE = str
CLKFBOUT_PHASE__RAW = str
SDCLK_PHASE = float
SDCLK_PHASE__TYPE = str
SDCLK_PHASE__RAW = str
CLK_PHASE = float
CLK_PHASE__TYPE = str
CLK_PHASE__RAW = str
CLK_DIV_PHASE = float
CLK_DIV_PHASE__TYPE = str
CLK_DIV_PHASE__RAW = str
MCLK_PHASE = float
MCLK_PHASE__TYPE = str
MCLK_PHASE__RAW = str
REF_JITTER1 = float
REF_JITTER1__TYPE = str
REF_JITTER1__RAW = str
SS_EN = str
SS_EN__TYPE = str
SS_EN__RAW = str
SS_MODE = str
SS_MODE__TYPE = str
SS_MODE__RAW = str
SS_MOD_PERIOD = int
SS_MOD_PERIOD__TYPE = str
SS_MOD_PERIOD__RAW = str
CMD_PAUSE_BITS = int
CMD_PAUSE_BITS__TYPE = str
CMD_PAUSE_BITS__RAW = str
CMD_DONE_BIT = int
CMD_DONE_BIT__TYPE = str
CMD_DONE_BIT__RAW = str
NUM_CYCLES_LOW_BIT = int
NUM_CYCLES_LOW_BIT__TYPE = str
NUM_CYCLES_LOW_BIT__RAW = str
NUM_CYCLES_00 = int
NUM_CYCLES_00__TYPE = str
NUM_CYCLES_00__RAW = str
NUM_CYCLES_01 = int
NUM_CYCLES_01__TYPE = str
NUM_CYCLES_01__RAW = str
NUM_CYCLES_02 = int
NUM_CYCLES_02__TYPE = str
NUM_CYCLES_02__RAW = str
NUM_CYCLES_03 = int
NUM_CYCLES_03__TYPE = str
NUM_CYCLES_03__RAW = str
NUM_CYCLES_04 = int
NUM_CYCLES_04__TYPE = str
NUM_CYCLES_04__RAW = str
NUM_CYCLES_05 = int
NUM_CYCLES_05__TYPE = str
NUM_CYCLES_05__RAW = str
NUM_CYCLES_06 = int
NUM_CYCLES_06__TYPE = str
NUM_CYCLES_06__RAW = str
NUM_CYCLES_07 = int
NUM_CYCLES_07__TYPE = str
NUM_CYCLES_07__RAW = str
NUM_CYCLES_08 = int
NUM_CYCLES_08__TYPE = str
NUM_CYCLES_08__RAW = str
NUM_CYCLES_09 = int
NUM_CYCLES_09__TYPE = str
NUM_CYCLES_09__RAW = str
NUM_CYCLES_10 = int
NUM_CYCLES_10__TYPE = str
NUM_CYCLES_10__RAW = str
NUM_CYCLES_11 = int
NUM_CYCLES_11__TYPE = str
NUM_CYCLES_11__RAW = str
NUM_CYCLES_12 = int
NUM_CYCLES_12__TYPE = str
NUM_CYCLES_12__RAW = str
NUM_CYCLES_13 = int
NUM_CYCLES_13__TYPE = str
NUM_CYCLES_13__RAW = str
NUM_CYCLES_14 = int
NUM_CYCLES_14__TYPE = str
NUM_CYCLES_14__RAW = str
NUM_CYCLES_15 = int
NUM_CYCLES_15__TYPE = str
NUM_CYCLES_15__RAW = str
NUM_CYCLES_16 = int
NUM_CYCLES_16__TYPE = str
NUM_CYCLES_16__RAW = str
NUM_CYCLES_17 = int
NUM_CYCLES_17__TYPE = str
NUM_CYCLES_17__RAW = str
NUM_CYCLES_18 = int
NUM_CYCLES_18__TYPE = str
NUM_CYCLES_18__RAW = str
NUM_CYCLES_19 = int
NUM_CYCLES_19__TYPE = str
NUM_CYCLES_19__RAW = str
NUM_CYCLES_20 = int
NUM_CYCLES_20__TYPE = str
NUM_CYCLES_20__RAW = str
NUM_CYCLES_21 = int
NUM_CYCLES_21__TYPE = str
NUM_CYCLES_21__RAW = str
NUM_CYCLES_22 = int
NUM_CYCLES_22__TYPE = str
NUM_CYCLES_22__RAW = str
NUM_CYCLES_23 = int
NUM_CYCLES_23__TYPE = str
NUM_CYCLES_23__RAW = str
NUM_CYCLES_24 = int
NUM_CYCLES_24__TYPE = str
NUM_CYCLES_24__RAW = str
NUM_CYCLES_25 = int
NUM_CYCLES_25__TYPE = str
NUM_CYCLES_25__RAW = str
NUM_CYCLES_26 = int
NUM_CYCLES_26__TYPE = str
NUM_CYCLES_26__RAW = str
NUM_CYCLES_27 = int
NUM_CYCLES_27__TYPE = str
NUM_CYCLES_27__RAW = str
NUM_CYCLES_28 = int
NUM_CYCLES_28__TYPE = str
NUM_CYCLES_28__RAW = str
NUM_CYCLES_29 = int
NUM_CYCLES_29__TYPE = str
NUM_CYCLES_29__RAW = str
NUM_CYCLES_30 = int
NUM_CYCLES_30__TYPE = str
NUM_CYCLES_30__RAW = str
NUM_CYCLES_31 = int
NUM_CYCLES_31__TYPE = str
NUM_CYCLES_31__RAW = str
MCNTRL_PS_ADDR = int
MCNTRL_PS_ADDR__TYPE = str
MCNTRL_PS_ADDR__RAW = str
MCNTRL_PS_MASK = int
MCNTRL_PS_MASK__TYPE = str
MCNTRL_PS_MASK__RAW = str
MCNTRL_PS_STATUS_REG_ADDR = int
MCNTRL_PS_STATUS_REG_ADDR__TYPE = str
MCNTRL_PS_STATUS_REG_ADDR__RAW = str
MCNTRL_PS_EN_RST = int
MCNTRL_PS_EN_RST__TYPE = str
MCNTRL_PS_EN_RST__RAW = str
MCNTRL_PS_CMD = int
MCNTRL_PS_CMD__TYPE = str
MCNTRL_PS_CMD__RAW = str
MCNTRL_PS_STATUS_CNTRL = int
MCNTRL_PS_STATUS_CNTRL__TYPE = str
MCNTRL_PS_STATUS_CNTRL__RAW = str
NUM_XFER_BITS = int
NUM_XFER_BITS__TYPE = str
NUM_XFER_BITS__RAW = str
FRAME_WIDTH_BITS = int
FRAME_WIDTH_BITS__TYPE = str
FRAME_WIDTH_BITS__RAW = str
FRAME_HEIGHT_BITS = int
FRAME_HEIGHT_BITS__TYPE = str
FRAME_HEIGHT_BITS__RAW = str
LAST_FRAME_BITS = int
LAST_FRAME_BITS__TYPE = str
LAST_FRAME_BITS__RAW = str
MCNTRL_SCANLINE_CHN1_ADDR = int
MCNTRL_SCANLINE_CHN1_ADDR__TYPE = str
MCNTRL_SCANLINE_CHN1_ADDR__RAW = str
MCNTRL_SCANLINE_CHN3_ADDR = int
MCNTRL_SCANLINE_CHN3_ADDR__TYPE = str
MCNTRL_SCANLINE_CHN3_ADDR__RAW = str
MCNTRL_SCANLINE_MASK = int
MCNTRL_SCANLINE_MASK__TYPE = str
MCNTRL_SCANLINE_MASK__RAW = str
MCNTRL_SCANLINE_MODE = int
MCNTRL_SCANLINE_MODE__TYPE = str
MCNTRL_SCANLINE_MODE__RAW = str
MCNTRL_SCANLINE_STATUS_CNTRL = int
MCNTRL_SCANLINE_STATUS_CNTRL__TYPE = str
MCNTRL_SCANLINE_STATUS_CNTRL__RAW = str
MCNTRL_SCANLINE_STARTADDR = int
MCNTRL_SCANLINE_STARTADDR__TYPE = str
MCNTRL_SCANLINE_STARTADDR__RAW = str
MCNTRL_SCANLINE_FRAME_SIZE = int
MCNTRL_SCANLINE_FRAME_SIZE__TYPE = str
MCNTRL_SCANLINE_FRAME_SIZE__RAW = str
MCNTRL_SCANLINE_FRAME_LAST = int
MCNTRL_SCANLINE_FRAME_LAST__TYPE = str
MCNTRL_SCANLINE_FRAME_LAST__RAW = str
MCNTRL_SCANLINE_FRAME_FULL_WIDTH = int
MCNTRL_SCANLINE_FRAME_FULL_WIDTH__TYPE = str
MCNTRL_SCANLINE_FRAME_FULL_WIDTH__RAW = str
MCNTRL_SCANLINE_WINDOW_WH = int
MCNTRL_SCANLINE_WINDOW_WH__TYPE = str
MCNTRL_SCANLINE_WINDOW_WH__RAW = str
MCNTRL_SCANLINE_WINDOW_X0Y0 = int
MCNTRL_SCANLINE_WINDOW_X0Y0__TYPE = str
MCNTRL_SCANLINE_WINDOW_X0Y0__RAW = str
MCNTRL_SCANLINE_WINDOW_STARTXY = int
MCNTRL_SCANLINE_WINDOW_STARTXY__TYPE = str
MCNTRL_SCANLINE_WINDOW_STARTXY__RAW = str
MCNTRL_SCANLINE_STATUS_REG_CHN1_ADDR = int
MCNTRL_SCANLINE_STATUS_REG_CHN1_ADDR__TYPE = str
MCNTRL_SCANLINE_STATUS_REG_CHN1_ADDR__RAW = str
MCNTRL_SCANLINE_STATUS_REG_CHN3_ADDR = int
MCNTRL_SCANLINE_STATUS_REG_CHN3_ADDR__TYPE = str
MCNTRL_SCANLINE_STATUS_REG_CHN3_ADDR__RAW = str
MCNTRL_SCANLINE_PENDING_CNTR_BITS = int
MCNTRL_SCANLINE_PENDING_CNTR_BITS__TYPE = str
MCNTRL_SCANLINE_PENDING_CNTR_BITS__RAW = str
MCNTRL_SCANLINE_FRAME_PAGE_RESET = int
MCNTRL_SCANLINE_FRAME_PAGE_RESET__TYPE = str
MCNTRL_SCANLINE_FRAME_PAGE_RESET__RAW = str
MAX_TILE_WIDTH = int
MAX_TILE_WIDTH__TYPE = str
MAX_TILE_WIDTH__RAW = str
MAX_TILE_HEIGHT = int
MAX_TILE_HEIGHT__TYPE = str
MAX_TILE_HEIGHT__RAW = str
MCNTRL_TILED_CHN2_ADDR = int
MCNTRL_TILED_CHN2_ADDR__TYPE = str
MCNTRL_TILED_CHN2_ADDR__RAW = str
MCNTRL_TILED_CHN4_ADDR = int
MCNTRL_TILED_CHN4_ADDR__TYPE = str
MCNTRL_TILED_CHN4_ADDR__RAW = str
MCNTRL_TILED_MASK = int
MCNTRL_TILED_MASK__TYPE = str
MCNTRL_TILED_MASK__RAW = str
MCNTRL_TILED_MODE = int
MCNTRL_TILED_MODE__TYPE = str
MCNTRL_TILED_MODE__RAW = str
MCNTRL_TILED_STATUS_CNTRL = int
MCNTRL_TILED_STATUS_CNTRL__TYPE = str
MCNTRL_TILED_STATUS_CNTRL__RAW = str
MCNTRL_TILED_STARTADDR = int
MCNTRL_TILED_STARTADDR__TYPE = str
MCNTRL_TILED_STARTADDR__RAW = str
MCNTRL_TILED_FRAME_SIZE = int
MCNTRL_TILED_FRAME_SIZE__TYPE = str
MCNTRL_TILED_FRAME_SIZE__RAW = str
MCNTRL_TILED_FRAME_LAST = int
MCNTRL_TILED_FRAME_LAST__TYPE = str
MCNTRL_TILED_FRAME_LAST__RAW = str
MCNTRL_TILED_FRAME_FULL_WIDTH = int
MCNTRL_TILED_FRAME_FULL_WIDTH__TYPE = str
MCNTRL_TILED_FRAME_FULL_WIDTH__RAW = str
MCNTRL_TILED_WINDOW_WH = int
MCNTRL_TILED_WINDOW_WH__TYPE = str
MCNTRL_TILED_WINDOW_WH__RAW = str
MCNTRL_TILED_WINDOW_X0Y0 = int
MCNTRL_TILED_WINDOW_X0Y0__TYPE = str
MCNTRL_TILED_WINDOW_X0Y0__RAW = str
MCNTRL_TILED_WINDOW_STARTXY = int
MCNTRL_TILED_WINDOW_STARTXY__TYPE = str
MCNTRL_TILED_WINDOW_STARTXY__RAW = str
MCNTRL_TILED_TILE_WHS = int
MCNTRL_TILED_TILE_WHS__TYPE = str
MCNTRL_TILED_TILE_WHS__RAW = str
MCNTRL_SCANLINE_START_DELAY = int
MCNTRL_SCANLINE_START_DELAY__TYPE = str
MCNTRL_SCANLINE_START_DELAY__RAW = str
MCNTRL_TILED_STATUS_REG_CHN2_ADDR = int
MCNTRL_TILED_STATUS_REG_CHN2_ADDR__TYPE = str
MCNTRL_TILED_STATUS_REG_CHN2_ADDR__RAW = str
MCNTRL_TILED_STATUS_REG_CHN4_ADDR = int
MCNTRL_TILED_STATUS_REG_CHN4_ADDR__TYPE = str
MCNTRL_TILED_STATUS_REG_CHN4_ADDR__RAW = str
MCNTRL_TILED_PENDING_CNTR_BITS = int
MCNTRL_TILED_PENDING_CNTR_BITS__TYPE = str
MCNTRL_TILED_PENDING_CNTR_BITS__RAW = str
MCNTRL_TILED_FRAME_PAGE_RESET = int
MCNTRL_TILED_FRAME_PAGE_RESET__TYPE = str
MCNTRL_TILED_FRAME_PAGE_RESET__RAW = str
BUFFER_DEPTH32 = int
BUFFER_DEPTH32__TYPE = str
BUFFER_DEPTH32__RAW = str
MCONTR_LINTILE_NRESET = int
MCONTR_LINTILE_NRESET__TYPE = str
MCONTR_LINTILE_NRESET__RAW = str
MCONTR_LINTILE_EN = int
MCONTR_LINTILE_EN__TYPE = str
MCONTR_LINTILE_EN__RAW = str
MCONTR_LINTILE_WRITE = int
MCONTR_LINTILE_WRITE__TYPE = str
MCONTR_LINTILE_WRITE__RAW = str
MCONTR_LINTILE_EXTRAPG = int
MCONTR_LINTILE_EXTRAPG__TYPE = str
MCONTR_LINTILE_EXTRAPG__RAW = str
MCONTR_LINTILE_EXTRAPG_BITS = int
MCONTR_LINTILE_EXTRAPG_BITS__TYPE = str
MCONTR_LINTILE_EXTRAPG_BITS__RAW = str
MCONTR_LINTILE_KEEP_OPEN = int
MCONTR_LINTILE_KEEP_OPEN__TYPE = str
MCONTR_LINTILE_KEEP_OPEN__RAW = str
MCONTR_LINTILE_BYTE32 = int
MCONTR_LINTILE_BYTE32__TYPE = str
MCONTR_LINTILE_BYTE32__RAW = str
MCONTR_LINTILE_LINEAR = int
MCONTR_LINTILE_LINEAR__TYPE = str
MCONTR_LINTILE_LINEAR__RAW = str
MCONTR_LINTILE_RST_FRAME = int
MCONTR_LINTILE_RST_FRAME__TYPE = str
MCONTR_LINTILE_RST_FRAME__RAW = str
MCONTR_LINTILE_SINGLE = int
MCONTR_LINTILE_SINGLE__TYPE = str
MCONTR_LINTILE_SINGLE__RAW = str
MCONTR_LINTILE_REPEAT = int
MCONTR_LINTILE_REPEAT__TYPE = str
MCONTR_LINTILE_REPEAT__RAW = str
MCONTR_LINTILE_DIS_NEED = int
MCONTR_LINTILE_DIS_NEED__TYPE = str
MCONTR_LINTILE_DIS_NEED__RAW = str
MCONTR_LINTILE_SKIP_LATE = int
MCONTR_LINTILE_SKIP_LATE__TYPE = str
MCONTR_LINTILE_SKIP_LATE__RAW = str
MCONTR_LINTILE_COPY_FRAME = int
MCONTR_LINTILE_COPY_FRAME__TYPE = str
MCONTR_LINTILE_COPY_FRAME__RAW = str
MCONTR_LINTILE_ABORT_LATE = int
MCONTR_LINTILE_ABORT_LATE__TYPE = str
MCONTR_LINTILE_ABORT_LATE__RAW = str
MCNTRL_SCANLINE_DLY_WIDTH = int
MCNTRL_SCANLINE_DLY_WIDTH__TYPE = str
MCNTRL_SCANLINE_DLY_WIDTH__RAW = str
MCNTRL_SCANLINE_DLY_DEFAULT = int
MCNTRL_SCANLINE_DLY_DEFAULT__TYPE = str
MCNTRL_SCANLINE_DLY_DEFAULT__RAW = str
MCNTRL_TEST01_ADDR = int
MCNTRL_TEST01_ADDR__TYPE = str
MCNTRL_TEST01_ADDR__RAW = str
MCNTRL_TEST01_MASK = int
MCNTRL_TEST01_MASK__TYPE = str
MCNTRL_TEST01_MASK__RAW = str
MCNTRL_TEST01_CHN1_MODE = int
MCNTRL_TEST01_CHN1_MODE__TYPE = str
MCNTRL_TEST01_CHN1_MODE__RAW = str
MCNTRL_TEST01_CHN1_STATUS_CNTRL = int
MCNTRL_TEST01_CHN1_STATUS_CNTRL__TYPE = str
MCNTRL_TEST01_CHN1_STATUS_CNTRL__RAW = str
MCNTRL_TEST01_CHN2_MODE = int
MCNTRL_TEST01_CHN2_MODE__TYPE = str
MCNTRL_TEST01_CHN2_MODE__RAW = str
MCNTRL_TEST01_CHN2_STATUS_CNTRL = int
MCNTRL_TEST01_CHN2_STATUS_CNTRL__TYPE = str
MCNTRL_TEST01_CHN2_STATUS_CNTRL__RAW = str
MCNTRL_TEST01_CHN3_MODE = int
MCNTRL_TEST01_CHN3_MODE__TYPE = str
MCNTRL_TEST01_CHN3_MODE__RAW = str
MCNTRL_TEST01_CHN3_STATUS_CNTRL = int
MCNTRL_TEST01_CHN3_STATUS_CNTRL__TYPE = str
MCNTRL_TEST01_CHN3_STATUS_CNTRL__RAW = str
MCNTRL_TEST01_CHN4_MODE = int
MCNTRL_TEST01_CHN4_MODE__TYPE = str
MCNTRL_TEST01_CHN4_MODE__RAW = str
MCNTRL_TEST01_CHN4_STATUS_CNTRL = int
MCNTRL_TEST01_CHN4_STATUS_CNTRL__TYPE = str
MCNTRL_TEST01_CHN4_STATUS_CNTRL__RAW = str
MCNTRL_TEST01_STATUS_REG_CHN1_ADDR = int
MCNTRL_TEST01_STATUS_REG_CHN1_ADDR__TYPE = str
MCNTRL_TEST01_STATUS_REG_CHN1_ADDR__RAW = str
MCNTRL_TEST01_STATUS_REG_CHN2_ADDR = int
MCNTRL_TEST01_STATUS_REG_CHN2_ADDR__TYPE = str
MCNTRL_TEST01_STATUS_REG_CHN2_ADDR__RAW = str
MCNTRL_TEST01_STATUS_REG_CHN3_ADDR = int
MCNTRL_TEST01_STATUS_REG_CHN3_ADDR__TYPE = str
MCNTRL_TEST01_STATUS_REG_CHN3_ADDR__RAW = str
MCNTRL_TEST01_STATUS_REG_CHN4_ADDR = int
MCNTRL_TEST01_STATUS_REG_CHN4_ADDR__TYPE = str
MCNTRL_TEST01_STATUS_REG_CHN4_ADDR__RAW = str
MCONTR_SENS_BASE = int
MCONTR_SENS_BASE__TYPE = str
MCONTR_SENS_BASE__RAW = str
MCONTR_SENS_INC = int
MCONTR_SENS_INC__TYPE = str
MCONTR_SENS_INC__RAW = str
MCONTR_CMPRS_BASE = int
MCONTR_CMPRS_BASE__TYPE = str
MCONTR_CMPRS_BASE__RAW = str
MCONTR_CMPRS_INC = int
MCONTR_CMPRS_INC__TYPE = str
MCONTR_CMPRS_INC__RAW = str
MCONTR_SENS_STATUS_BASE = int
MCONTR_SENS_STATUS_BASE__TYPE = str
MCONTR_SENS_STATUS_BASE__RAW = str
MCONTR_SENS_STATUS_INC = int
MCONTR_SENS_STATUS_INC__TYPE = str
MCONTR_SENS_STATUS_INC__RAW = str
MCONTR_CMPRS_STATUS_BASE = int
MCONTR_CMPRS_STATUS_BASE__TYPE = str
MCONTR_CMPRS_STATUS_BASE__RAW = str
MCONTR_CMPRS_STATUS_INC = int
MCONTR_CMPRS_STATUS_INC__TYPE = str
MCONTR_CMPRS_STATUS_INC__RAW = str
MEMBRIDGE_ADDR = int
MEMBRIDGE_ADDR__TYPE = str
MEMBRIDGE_ADDR__RAW = str
MEMBRIDGE_MASK = int
MEMBRIDGE_MASK__TYPE = str
MEMBRIDGE_MASK__RAW = str
MEMBRIDGE_CTRL = int
MEMBRIDGE_CTRL__TYPE = str
MEMBRIDGE_CTRL__RAW = str
MEMBRIDGE_STATUS_CNTRL = int
MEMBRIDGE_STATUS_CNTRL__TYPE = str
MEMBRIDGE_STATUS_CNTRL__RAW = str
MEMBRIDGE_LO_ADDR64 = int
MEMBRIDGE_LO_ADDR64__TYPE = str
MEMBRIDGE_LO_ADDR64__RAW = str
MEMBRIDGE_SIZE64 = int
MEMBRIDGE_SIZE64__TYPE = str
MEMBRIDGE_SIZE64__RAW = str
MEMBRIDGE_START64 = int
MEMBRIDGE_START64__TYPE = str
MEMBRIDGE_START64__RAW = str
MEMBRIDGE_LEN64 = int
MEMBRIDGE_LEN64__TYPE = str
MEMBRIDGE_LEN64__RAW = str
MEMBRIDGE_WIDTH64 = int
MEMBRIDGE_WIDTH64__TYPE = str
MEMBRIDGE_WIDTH64__RAW = str
MEMBRIDGE_CTRL_IRQ = int
MEMBRIDGE_CTRL_IRQ__TYPE = str
MEMBRIDGE_CTRL_IRQ__RAW = str
MEMBRIDGE_STATUS_REG = int
MEMBRIDGE_STATUS_REG__TYPE = str
MEMBRIDGE_STATUS_REG__RAW = str
RSEL = int
RSEL__TYPE = str
RSEL__RAW = str
WSEL = int
WSEL__TYPE = str
WSEL__RAW = str
SENSOR_GROUP_ADDR = int
SENSOR_GROUP_ADDR__TYPE = str
SENSOR_GROUP_ADDR__RAW = str
SENSOR_BASE_INC = int
SENSOR_BASE_INC__TYPE = str
SENSOR_BASE_INC__RAW = str
HIST_SAXI_ADDR_REL = int
HIST_SAXI_ADDR_REL__TYPE = str
HIST_SAXI_ADDR_REL__RAW = str
HIST_SAXI_MODE_ADDR_REL = int
HIST_SAXI_MODE_ADDR_REL__TYPE = str
HIST_SAXI_MODE_ADDR_REL__RAW = str
SENSI2C_STATUS_REG_BASE = int
SENSI2C_STATUS_REG_BASE__TYPE = str
SENSI2C_STATUS_REG_BASE__RAW = str
SENSI2C_STATUS_REG_INC = int
SENSI2C_STATUS_REG_INC__TYPE = str
SENSI2C_STATUS_REG_INC__RAW = str
SENSI2C_STATUS_REG_REL = int
SENSI2C_STATUS_REG_REL__TYPE = str
SENSI2C_STATUS_REG_REL__RAW = str
SENSIO_STATUS_REG_REL = int
SENSIO_STATUS_REG_REL__TYPE = str
SENSIO_STATUS_REG_REL__RAW = str
SENSOR_TIMING_STATUS_REG_BASE = int
SENSOR_TIMING_STATUS_REG_BASE__TYPE = str
SENSOR_TIMING_STATUS_REG_BASE__RAW = str
SENSOR_TIMING_STATUS_REG_INC = int
SENSOR_TIMING_STATUS_REG_INC__TYPE = str
SENSOR_TIMING_STATUS_REG_INC__RAW = str
SENSOR_TIMING_BITS = int
SENSOR_TIMING_BITS__TYPE = str
SENSOR_TIMING_BITS__RAW = str
SENSOR_TIMING_START = int
SENSOR_TIMING_START__TYPE = str
SENSOR_TIMING_START__RAW = str
SENSOR_TIMING_LANE = int
SENSOR_TIMING_LANE__TYPE = str
SENSOR_TIMING_LANE__RAW = str
SENSOR_TIMING_FROM = int
SENSOR_TIMING_FROM__TYPE = str
SENSOR_TIMING_FROM__RAW = str
SENSOR_TIMING_TO = int
SENSOR_TIMING_TO__TYPE = str
SENSOR_TIMING_TO__RAW = str
SENSOR_NUM_HISTOGRAM = int
SENSOR_NUM_HISTOGRAM__TYPE = str
SENSOR_NUM_HISTOGRAM__RAW = str
HISTOGRAM_RAM_MODE = str
HISTOGRAM_RAM_MODE__TYPE = str
HISTOGRAM_RAM_MODE__RAW = str
SENS_NUM_SUBCHN = int
SENS_NUM_SUBCHN__TYPE = str
SENS_NUM_SUBCHN__RAW = str
SENS_GAMMA_BUFFER = int
SENS_GAMMA_BUFFER__TYPE = str
SENS_GAMMA_BUFFER__RAW = str
SENSOR_CTRL_RADDR = int
SENSOR_CTRL_RADDR__TYPE = str
SENSOR_CTRL_RADDR__RAW = str
SENSOR_CTRL_ADDR_MASK = int
SENSOR_CTRL_ADDR_MASK__TYPE = str
SENSOR_CTRL_ADDR_MASK__RAW = str
SENSOR_HIST_EN_BITS = int
SENSOR_HIST_EN_BITS__TYPE = str
SENSOR_HIST_EN_BITS__RAW = str
SENSOR_HIST_NRST_BITS = int
SENSOR_HIST_NRST_BITS__TYPE = str
SENSOR_HIST_NRST_BITS__RAW = str
SENSOR_HIST_BITS_SET = int
SENSOR_HIST_BITS_SET__TYPE = str
SENSOR_HIST_BITS_SET__RAW = str
SENSOR_CHN_EN_BIT = int
SENSOR_CHN_EN_BIT__TYPE = str
SENSOR_CHN_EN_BIT__RAW = str
SENSOR_CHN_EN_BIT_SET = int
SENSOR_CHN_EN_BIT_SET__TYPE = str
SENSOR_CHN_EN_BIT_SET__RAW = str
SENSOR_16BIT_BIT = int
SENSOR_16BIT_BIT__TYPE = str
SENSOR_16BIT_BIT__RAW = str
SENSOR_16BIT_BIT_SET = int
SENSOR_16BIT_BIT_SET__TYPE = str
SENSOR_16BIT_BIT_SET__RAW = str
SENSI2C_CTRL_RADDR = int
SENSI2C_CTRL_RADDR__TYPE = str
SENSI2C_CTRL_RADDR__RAW = str
SENSI2C_CTRL_MASK = int
SENSI2C_CTRL_MASK__TYPE = str
SENSI2C_CTRL_MASK__RAW = str
SENSI2C_CTRL = int
SENSI2C_CTRL__TYPE = str
SENSI2C_CTRL__RAW = str
SENSI2C_CMD_TABLE = int
SENSI2C_CMD_TABLE__TYPE = str
SENSI2C_CMD_TABLE__RAW = str
SENSI2C_CMD_TAND = int
SENSI2C_CMD_TAND__TYPE = str
SENSI2C_CMD_TAND__RAW = str
SENSI2C_CMD_RESET = int
SENSI2C_CMD_RESET__TYPE = str
SENSI2C_CMD_RESET__RAW = str
SENSI2C_CMD_RUN = int
SENSI2C_CMD_RUN__TYPE = str
SENSI2C_CMD_RUN__RAW = str
SENSI2C_CMD_RUN_PBITS = int
SENSI2C_CMD_RUN_PBITS__TYPE = str
SENSI2C_CMD_RUN_PBITS__RAW = str
SENSI2C_CMD_USE_EOF = int
SENSI2C_CMD_USE_EOF__TYPE = str
SENSI2C_CMD_USE_EOF__RAW = str
SENSI2C_CMD_SOFT_SDA = int
SENSI2C_CMD_SOFT_SDA__TYPE = str
SENSI2C_CMD_SOFT_SDA__RAW = str
SENSI2C_CMD_SOFT_SCL = int
SENSI2C_CMD_SOFT_SCL__TYPE = str
SENSI2C_CMD_SOFT_SCL__RAW = str
SENSI2C_CMD_FIFO_RD = int
SENSI2C_CMD_FIFO_RD__TYPE = str
SENSI2C_CMD_FIFO_RD__RAW = str
SENSI2C_CMD_ACIVE = int
SENSI2C_CMD_ACIVE__TYPE = str
SENSI2C_CMD_ACIVE__RAW = str
SENSI2C_CMD_ACIVE_EARLY0 = int
SENSI2C_CMD_ACIVE_EARLY0__TYPE = str
SENSI2C_CMD_ACIVE_EARLY0__RAW = str
SENSI2C_CMD_ACIVE_SDA = int
SENSI2C_CMD_ACIVE_SDA__TYPE = str
SENSI2C_CMD_ACIVE_SDA__RAW = str
SENSI2C_TBL_RAH = int
SENSI2C_TBL_RAH__TYPE = str
SENSI2C_TBL_RAH__RAW = str
SENSI2C_TBL_RAH_BITS = int
SENSI2C_TBL_RAH_BITS__TYPE = str
SENSI2C_TBL_RAH_BITS__RAW = str
SENSI2C_TBL_RNWREG = int
SENSI2C_TBL_RNWREG__TYPE = str
SENSI2C_TBL_RNWREG__RAW = str
SENSI2C_TBL_SA = int
SENSI2C_TBL_SA__TYPE = str
SENSI2C_TBL_SA__RAW = str
SENSI2C_TBL_SA_BITS = int
SENSI2C_TBL_SA_BITS__TYPE = str
SENSI2C_TBL_SA_BITS__RAW = str
SENSI2C_TBL_NBWR = int
SENSI2C_TBL_NBWR__TYPE = str
SENSI2C_TBL_NBWR__RAW = str
SENSI2C_TBL_NBWR_BITS = int
SENSI2C_TBL_NBWR_BITS__TYPE = str
SENSI2C_TBL_NBWR_BITS__RAW = str
SENSI2C_TBL_NBRD = int
SENSI2C_TBL_NBRD__TYPE = str
SENSI2C_TBL_NBRD__RAW = str
SENSI2C_TBL_NBRD_BITS = int
SENSI2C_TBL_NBRD_BITS__TYPE = str
SENSI2C_TBL_NBRD_BITS__RAW = str
SENSI2C_TBL_NABRD = int
SENSI2C_TBL_NABRD__TYPE = str
SENSI2C_TBL_NABRD__RAW = str
SENSI2C_TBL_DLY = int
SENSI2C_TBL_DLY__TYPE = str
SENSI2C_TBL_DLY__RAW = str
SENSI2C_TBL_DLY_BITS = int
SENSI2C_TBL_DLY_BITS__TYPE = str
SENSI2C_TBL_DLY_BITS__RAW = str
SENSI2C_STATUS = int
SENSI2C_STATUS__TYPE = str
SENSI2C_STATUS__RAW = str
SENS_SYNC_RADDR = int
SENS_SYNC_RADDR__TYPE = str
SENS_SYNC_RADDR__RAW = str
SENS_SYNC_MASK = int
SENS_SYNC_MASK__TYPE = str
SENS_SYNC_MASK__RAW = str
SENS_SYNC_MULT = int
SENS_SYNC_MULT__TYPE = str
SENS_SYNC_MULT__RAW = str
SENS_SYNC_LATE = int
SENS_SYNC_LATE__TYPE = str
SENS_SYNC_LATE__RAW = str
SENS_GAMMA_RADDR = int
SENS_GAMMA_RADDR__TYPE = str
SENS_GAMMA_RADDR__RAW = str
SENS_GAMMA_ADDR_MASK = int
SENS_GAMMA_ADDR_MASK__TYPE = str
SENS_GAMMA_ADDR_MASK__RAW = str
SENS_GAMMA_CTRL = int
SENS_GAMMA_CTRL__TYPE = str
SENS_GAMMA_CTRL__RAW = str
SENS_GAMMA_ADDR_DATA = int
SENS_GAMMA_ADDR_DATA__TYPE = str
SENS_GAMMA_ADDR_DATA__RAW = str
SENS_GAMMA_HEIGHT01 = int
SENS_GAMMA_HEIGHT01__TYPE = str
SENS_GAMMA_HEIGHT01__RAW = str
SENS_GAMMA_HEIGHT2 = int
SENS_GAMMA_HEIGHT2__TYPE = str
SENS_GAMMA_HEIGHT2__RAW = str
SENS_GAMMA_MODE_BAYER = int
SENS_GAMMA_MODE_BAYER__TYPE = str
SENS_GAMMA_MODE_BAYER__RAW = str
SENS_GAMMA_MODE_BAYER_SET = int
SENS_GAMMA_MODE_BAYER_SET__TYPE = str
SENS_GAMMA_MODE_BAYER_SET__RAW = str
SENS_GAMMA_MODE_PAGE = int
SENS_GAMMA_MODE_PAGE__TYPE = str
SENS_GAMMA_MODE_PAGE__RAW = str
SENS_GAMMA_MODE_PAGE_SET = int
SENS_GAMMA_MODE_PAGE_SET__TYPE = str
SENS_GAMMA_MODE_PAGE_SET__RAW = str
SENS_GAMMA_MODE_EN = int
SENS_GAMMA_MODE_EN__TYPE = str
SENS_GAMMA_MODE_EN__RAW = str
SENS_GAMMA_MODE_EN_SET = int
SENS_GAMMA_MODE_EN_SET__TYPE = str
SENS_GAMMA_MODE_EN_SET__RAW = str
SENS_GAMMA_MODE_REPET = int
SENS_GAMMA_MODE_REPET__TYPE = str
SENS_GAMMA_MODE_REPET__RAW = str
SENS_GAMMA_MODE_REPET_SET = int
SENS_GAMMA_MODE_REPET_SET__TYPE = str
SENS_GAMMA_MODE_REPET_SET__RAW = str
SENS_GAMMA_MODE_TRIG = int
SENS_GAMMA_MODE_TRIG__TYPE = str
SENS_GAMMA_MODE_TRIG__RAW = str
SENS_LENS_RADDR = int
SENS_LENS_RADDR__TYPE = str
SENS_LENS_RADDR__RAW = str
SENS_LENS_ADDR_MASK = int
SENS_LENS_ADDR_MASK__TYPE = str
SENS_LENS_ADDR_MASK__RAW = str
SENS_LENS_COEFF = int
SENS_LENS_COEFF__TYPE = str
SENS_LENS_COEFF__RAW = str
SENS_LENS_AX = int
SENS_LENS_AX__TYPE = str
SENS_LENS_AX__RAW = str
SENS_LENS_AX_MASK = int
SENS_LENS_AX_MASK__TYPE = str
SENS_LENS_AX_MASK__RAW = str
SENS_LENS_AY = int
SENS_LENS_AY__TYPE = str
SENS_LENS_AY__RAW = str
SENS_LENS_AY_MASK = int
SENS_LENS_AY_MASK__TYPE = str
SENS_LENS_AY_MASK__RAW = str
SENS_LENS_C = int
SENS_LENS_C__TYPE = str
SENS_LENS_C__RAW = str
SENS_LENS_C_MASK = int
SENS_LENS_C_MASK__TYPE = str
SENS_LENS_C_MASK__RAW = str
SENS_LENS_BX = int
SENS_LENS_BX__TYPE = str
SENS_LENS_BX__RAW = str
SENS_LENS_BX_MASK = int
SENS_LENS_BX_MASK__TYPE = str
SENS_LENS_BX_MASK__RAW = str
SENS_LENS_BY = int
SENS_LENS_BY__TYPE = str
SENS_LENS_BY__RAW = str
SENS_LENS_BY_MASK = int
SENS_LENS_BY_MASK__TYPE = str
SENS_LENS_BY_MASK__RAW = str
SENS_LENS_SCALES = int
SENS_LENS_SCALES__TYPE = str
SENS_LENS_SCALES__RAW = str
SENS_LENS_SCALES_MASK = int
SENS_LENS_SCALES_MASK__TYPE = str
SENS_LENS_SCALES_MASK__RAW = str
SENS_LENS_FAT0_IN = int
SENS_LENS_FAT0_IN__TYPE = str
SENS_LENS_FAT0_IN__RAW = str
SENS_LENS_FAT0_IN_MASK = int
SENS_LENS_FAT0_IN_MASK__TYPE = str
SENS_LENS_FAT0_IN_MASK__RAW = str
SENS_LENS_FAT0_OUT = int
SENS_LENS_FAT0_OUT__TYPE = str
SENS_LENS_FAT0_OUT__RAW = str
SENS_LENS_FAT0_OUT_MASK = int
SENS_LENS_FAT0_OUT_MASK__TYPE = str
SENS_LENS_FAT0_OUT_MASK__RAW = str
SENS_LENS_POST_SCALE = int
SENS_LENS_POST_SCALE__TYPE = str
SENS_LENS_POST_SCALE__RAW = str
SENS_LENS_POST_SCALE_MASK = int
SENS_LENS_POST_SCALE_MASK__TYPE = str
SENS_LENS_POST_SCALE_MASK__RAW = str
SENSIO_RADDR = int
SENSIO_RADDR__TYPE = str
SENSIO_RADDR__RAW = str
SENSIO_ADDR_MASK = int
SENSIO_ADDR_MASK__TYPE = str
SENSIO_ADDR_MASK__RAW = str
SENSIO_CTRL = int
SENSIO_CTRL__TYPE = str
SENSIO_CTRL__RAW = str
SENS_CTRL_MRST = int
SENS_CTRL_MRST__TYPE = str
SENS_CTRL_MRST__RAW = str
SENS_CTRL_ARST = int
SENS_CTRL_ARST__TYPE = str
SENS_CTRL_ARST__RAW = str
SENS_CTRL_ARO = int
SENS_CTRL_ARO__TYPE = str
SENS_CTRL_ARO__RAW = str
SENS_CTRL_RST_MMCM = int
SENS_CTRL_RST_MMCM__TYPE = str
SENS_CTRL_RST_MMCM__RAW = str
SENS_CTRL_IGNORE_EMBED = int
SENS_CTRL_IGNORE_EMBED__TYPE = str
SENS_CTRL_IGNORE_EMBED__RAW = str
SENS_CTRL_EXT_CLK = int
SENS_CTRL_EXT_CLK__TYPE = str
SENS_CTRL_EXT_CLK__RAW = str
SENS_CTRL_LD_DLY = int
SENS_CTRL_LD_DLY__TYPE = str
SENS_CTRL_LD_DLY__RAW = str
SENS_CTRL_GP0 = int
SENS_CTRL_GP0__TYPE = str
SENS_CTRL_GP0__RAW = str
SENS_CTRL_GP1 = int
SENS_CTRL_GP1__TYPE = str
SENS_CTRL_GP1__RAW = str
SENS_CTRL_GP2 = int
SENS_CTRL_GP2__TYPE = str
SENS_CTRL_GP2__RAW = str
SENS_CTRL_GP3 = int
SENS_CTRL_GP3__TYPE = str
SENS_CTRL_GP3__RAW = str
SENS_UART_EXTIF_EN = int
SENS_UART_EXTIF_EN__TYPE = str
SENS_UART_EXTIF_EN__RAW = str
SENS_UART_XMIT_RST = int
SENS_UART_XMIT_RST__TYPE = str
SENS_UART_XMIT_RST__RAW = str
SENS_UART_RECV_RST = int
SENS_UART_RECV_RST__TYPE = str
SENS_UART_RECV_RST__RAW = str
SENS_UART_XMIT_START = int
SENS_UART_XMIT_START__TYPE = str
SENS_UART_XMIT_START__RAW = str
SENS_UART_RECV_NEXT = int
SENS_UART_RECV_NEXT__TYPE = str
SENS_UART_RECV_NEXT__RAW = str
SENS_CTRL_QUADRANTS = int
SENS_CTRL_QUADRANTS__TYPE = str
SENS_CTRL_QUADRANTS__RAW = str
SENS_CTRL_QUADRANTS_WIDTH = int
SENS_CTRL_QUADRANTS_WIDTH__TYPE = str
SENS_CTRL_QUADRANTS_WIDTH__RAW = str
SENS_CTRL_ODD = int
SENS_CTRL_ODD__TYPE = str
SENS_CTRL_ODD__RAW = str
SENS_CTRL_QUADRANTS_EN = int
SENS_CTRL_QUADRANTS_EN__TYPE = str
SENS_CTRL_QUADRANTS_EN__RAW = str
SENSIO_STATUS = int
SENSIO_STATUS__TYPE = str
SENSIO_STATUS__RAW = str
SENSIO_JTAG = int
SENSIO_JTAG__TYPE = str
SENSIO_JTAG__RAW = str
SENS_JTAG_PGMEN = int
SENS_JTAG_PGMEN__TYPE = str
SENS_JTAG_PGMEN__RAW = str
SENS_JTAG_PROG = int
SENS_JTAG_PROG__TYPE = str
SENS_JTAG_PROG__RAW = str
SENS_JTAG_TCK = int
SENS_JTAG_TCK__TYPE = str
SENS_JTAG_TCK__RAW = str
SENS_JTAG_TMS = int
SENS_JTAG_TMS__TYPE = str
SENS_JTAG_TMS__RAW = str
SENS_JTAG_TDI = int
SENS_JTAG_TDI__TYPE = str
SENS_JTAG_TDI__RAW = str
SENSIO_WIDTH = int
SENSIO_WIDTH__TYPE = str
SENSIO_WIDTH__RAW = str
SENSIO_DELAYS = int
SENSIO_DELAYS__TYPE = str
SENSIO_DELAYS__RAW = str
SENSI2C_ABS_RADDR = int
SENSI2C_ABS_RADDR__TYPE = str
SENSI2C_ABS_RADDR__RAW = str
SENSI2C_REL_RADDR = int
SENSI2C_REL_RADDR__TYPE = str
SENSI2C_REL_RADDR__RAW = str
SENSI2C_ADDR_MASK = int
SENSI2C_ADDR_MASK__TYPE = str
SENSI2C_ADDR_MASK__RAW = str
HISTOGRAM_RADDR_INC = int
HISTOGRAM_RADDR_INC__TYPE = str
HISTOGRAM_RADDR_INC__RAW = str
HISTOGRAM_RADDR0 = int
HISTOGRAM_RADDR0__TYPE = str
HISTOGRAM_RADDR0__RAW = str
HISTOGRAM_RADDR1 = int
HISTOGRAM_RADDR1__TYPE = str
HISTOGRAM_RADDR1__RAW = str
HISTOGRAM_RADDR2 = int
HISTOGRAM_RADDR2__TYPE = str
HISTOGRAM_RADDR2__RAW = str
HISTOGRAM_RADDR3 = int
HISTOGRAM_RADDR3__TYPE = str
HISTOGRAM_RADDR3__RAW = str
HISTOGRAM_ADDR_MASK = int
HISTOGRAM_ADDR_MASK__TYPE = str
HISTOGRAM_ADDR_MASK__RAW = str
HISTOGRAM_LEFT_TOP = int
HISTOGRAM_LEFT_TOP__TYPE = str
HISTOGRAM_LEFT_TOP__RAW = str
HISTOGRAM_WIDTH_HEIGHT = int
HISTOGRAM_WIDTH_HEIGHT__TYPE = str
HISTOGRAM_WIDTH_HEIGHT__RAW = str
XOR_HIST_BAYER = int
XOR_HIST_BAYER__TYPE = str
XOR_HIST_BAYER__RAW = str
SENSI2C_DRIVE = int
SENSI2C_DRIVE__TYPE = str
SENSI2C_DRIVE__RAW = str
SENSI2C_IBUF_LOW_PWR = str
SENSI2C_IBUF_LOW_PWR__TYPE = str
SENSI2C_IBUF_LOW_PWR__RAW = str
SENSI2C_SLEW = str
SENSI2C_SLEW__TYPE = str
SENSI2C_SLEW__RAW = str
VOSPI_MRST = int
VOSPI_MRST__TYPE = str
VOSPI_MRST__RAW = str
VOSPI_MRST_BITS = int
VOSPI_MRST_BITS__TYPE = str
VOSPI_MRST_BITS__RAW = str
VOSPI_RST_SEQ = int
VOSPI_RST_SEQ__TYPE = str
VOSPI_RST_SEQ__RAW = str
VOSPI_SPI_SEQ = int
VOSPI_SPI_SEQ__TYPE = str
VOSPI_SPI_SEQ__RAW = str
VOSPI_MCLK = int
VOSPI_MCLK__TYPE = str
VOSPI_MCLK__RAW = str
VOSPI_MCLK_BITS = int
VOSPI_MCLK_BITS__TYPE = str
VOSPI_MCLK_BITS__RAW = str
VOSPI_EN = int
VOSPI_EN__TYPE = str
VOSPI_EN__RAW = str
VOSPI_EN_BITS = int
VOSPI_EN_BITS__TYPE = str
VOSPI_EN_BITS__RAW = str
VOSPI_OUT_EN = int
VOSPI_OUT_EN__TYPE = str
VOSPI_OUT_EN__RAW = str
VOSPI_OUT_EN_BITS = int
VOSPI_OUT_EN_BITS__TYPE = str
VOSPI_OUT_EN_BITS__RAW = str
VOSPI_OUT_EN_SINGL = int
VOSPI_OUT_EN_SINGL__TYPE = str
VOSPI_OUT_EN_SINGL__RAW = str
VOSPI_RESET_ERR = int
VOSPI_RESET_ERR__TYPE = str
VOSPI_RESET_ERR__RAW = str
VOSPI_SPI_CLK = int
VOSPI_SPI_CLK__TYPE = str
VOSPI_SPI_CLK__RAW = str
VOSPI_SPI_CLK_BITS = int
VOSPI_SPI_CLK_BITS__TYPE = str
VOSPI_SPI_CLK_BITS__RAW = str
VOSPI_SEGM0_OK = int
VOSPI_SEGM0_OK__TYPE = str
VOSPI_SEGM0_OK__RAW = str
VOSPI_SEGM0_OK_BITS = int
VOSPI_SEGM0_OK_BITS__TYPE = str
VOSPI_SEGM0_OK_BITS__RAW = str
VOSPI_VSYNC = int
VOSPI_VSYNC__TYPE = str
VOSPI_VSYNC__RAW = str
VOSPI_VSYNC_BITS = int
VOSPI_VSYNC_BITS__TYPE = str
VOSPI_VSYNC_BITS__RAW = str
VOSPI_NORESYNC = int
VOSPI_NORESYNC__TYPE = str
VOSPI_NORESYNC__RAW = str
VOSPI_NORESYNC_BITS = int
VOSPI_NORESYNC_BITS__TYPE = str
VOSPI_NORESYNC_BITS__RAW = str
VOSPI_TELEMETRY = int
VOSPI_TELEMETRY__TYPE = str
VOSPI_TELEMETRY__RAW = str
VOSPI_TELEMETRY_BITS = int
VOSPI_TELEMETRY_BITS__TYPE = str
VOSPI_TELEMETRY_BITS__RAW = str
VOSPI_GPIO = int
VOSPI_GPIO__TYPE = str
VOSPI_GPIO__RAW = str
VOSPI_GPIO_BITS = int
VOSPI_GPIO_BITS__TYPE = str
VOSPI_GPIO_BITS__RAW = str
VOSPI_DBG_SRC = int
VOSPI_DBG_SRC__TYPE = str
VOSPI_DBG_SRC__RAW = str
VOSPI_DBG_SRC_BITS = int
VOSPI_DBG_SRC_BITS__TYPE = str
VOSPI_DBG_SRC_BITS__RAW = str
VOSPI_PACKET_WORDS = int
VOSPI_PACKET_WORDS__TYPE = str
VOSPI_PACKET_WORDS__RAW = str
VOSPI_NO_INVALID = int
VOSPI_NO_INVALID__TYPE = str
VOSPI_NO_INVALID__RAW = str
VOSPI_PACKETS_PER_LINE = int
VOSPI_PACKETS_PER_LINE__TYPE = str
VOSPI_PACKETS_PER_LINE__RAW = str
VOSPI_SEGMENT_FIRST = int
VOSPI_SEGMENT_FIRST__TYPE = str
VOSPI_SEGMENT_FIRST__RAW = str
VOSPI_SEGMENT_LAST = int
VOSPI_SEGMENT_LAST__TYPE = str
VOSPI_SEGMENT_LAST__RAW = str
VOSPI_PACKET_FIRST = int
VOSPI_PACKET_FIRST__TYPE = str
VOSPI_PACKET_FIRST__RAW = str
VOSPI_PACKET_LAST = int
VOSPI_PACKET_LAST__TYPE = str
VOSPI_PACKET_LAST__RAW = str
VOSPI_PACKET_TTT = int
VOSPI_PACKET_TTT__TYPE = str
VOSPI_PACKET_TTT__RAW = str
VOSPI_SOF_TO_HACT = int
VOSPI_SOF_TO_HACT__TYPE = str
VOSPI_SOF_TO_HACT__RAW = str
VOSPI_HACT_TO_HACT_EOF = int
VOSPI_HACT_TO_HACT_EOF__TYPE = str
VOSPI_HACT_TO_HACT_EOF__RAW = str
VOSPI_MCLK_HALFDIV = int
VOSPI_MCLK_HALFDIV__TYPE = str
VOSPI_MCLK_HALFDIV__RAW = str
VOSPI_MRST_MS = int
VOSPI_MRST_MS__TYPE = str
VOSPI_MRST_MS__RAW = str
VOSPI_MRST_AFTER_MS = int
VOSPI_MRST_AFTER_MS__TYPE = str
VOSPI_MRST_AFTER_MS__RAW = str
VOSPI_SPI_TIMEOUT_MS = int
VOSPI_SPI_TIMEOUT_MS__TYPE = str
VOSPI_SPI_TIMEOUT_MS__RAW = str
SENSOR_DATA_WIDTH = int
SENSOR_DATA_WIDTH__TYPE = str
SENSOR_DATA_WIDTH__RAW = str
SENSOR_FIFO_2DEPTH = int
SENSOR_FIFO_2DEPTH__TYPE = str
SENSOR_FIFO_2DEPTH__RAW = str
SENSOR_FIFO_DELAY = int
SENSOR_FIFO_DELAY__TYPE = str
SENSOR_FIFO_DELAY__RAW = str
HIST_SAXI_ADDR_MASK = int
HIST_SAXI_ADDR_MASK__TYPE = str
HIST_SAXI_ADDR_MASK__RAW = str
HIST_SAXI_MODE_WIDTH = int
HIST_SAXI_MODE_WIDTH__TYPE = str
HIST_SAXI_MODE_WIDTH__RAW = str
HIST_SAXI_EN = int
HIST_SAXI_EN__TYPE = str
HIST_SAXI_EN__RAW = str
HIST_SAXI_NRESET = int
HIST_SAXI_NRESET__TYPE = str
HIST_SAXI_NRESET__RAW = str
HIST_CONFIRM_WRITE = int
HIST_CONFIRM_WRITE__TYPE = str
HIST_CONFIRM_WRITE__RAW = str
HIST_SAXI_AWCACHE = int
HIST_SAXI_AWCACHE__TYPE = str
HIST_SAXI_AWCACHE__RAW = str
HIST_SAXI_MODE_ADDR_MASK = int
HIST_SAXI_MODE_ADDR_MASK__TYPE = str
HIST_SAXI_MODE_ADDR_MASK__RAW = str
NUM_FRAME_BITS = int
NUM_FRAME_BITS__TYPE = str
NUM_FRAME_BITS__RAW = str
SENS_SYNC_FBITS = int
SENS_SYNC_FBITS__TYPE = str
SENS_SYNC_FBITS__RAW = str
SENS_SYNC_LBITS = int
SENS_SYNC_LBITS__TYPE = str
SENS_SYNC_LBITS__RAW = str
SENS_SYNC_LATE_DFLT = int
SENS_SYNC_LATE_DFLT__TYPE = str
SENS_SYNC_LATE_DFLT__RAW = str
SENS_SYNC_MINBITS = int
SENS_SYNC_MINBITS__TYPE = str
SENS_SYNC_MINBITS__RAW = str
SENS_SYNC_MINPER = int
SENS_SYNC_MINPER__TYPE = str
SENS_SYNC_MINPER__RAW = str
IDELAY_VALUE = int
IDELAY_VALUE__TYPE = str
IDELAY_VALUE__RAW = str
PXD_DRIVE = int
PXD_DRIVE__TYPE = str
PXD_DRIVE__RAW = str
PXD_IBUF_LOW_PWR = str
PXD_IBUF_LOW_PWR__TYPE = str
PXD_IBUF_LOW_PWR__RAW = str
PXD_SLEW = str
PXD_SLEW__TYPE = str
PXD_SLEW__RAW = str
VOSPI_DRIVE = int
VOSPI_DRIVE__TYPE = str
VOSPI_DRIVE__RAW = str
VOSPI_IBUF_LOW_PWR = str
VOSPI_IBUF_LOW_PWR__TYPE = str
VOSPI_IBUF_LOW_PWR__RAW = str
VOSPI_IOSTANDARD = str
VOSPI_IOSTANDARD__TYPE = str
VOSPI_IOSTANDARD__RAW = str
VOSPI_SLEW = str
VOSPI_SLEW__TYPE = str
VOSPI_SLEW__RAW = str
SENS_REFCLK_FREQUENCY = float
SENS_REFCLK_FREQUENCY__TYPE = str
SENS_REFCLK_FREQUENCY__RAW = str
SENS_HIGH_PERFORMANCE_MODE = str
SENS_HIGH_PERFORMANCE_MODE__TYPE = str
SENS_HIGH_PERFORMANCE_MODE__RAW = str
PXD_CAPACITANCE = str
PXD_CAPACITANCE__TYPE = str
PXD_CAPACITANCE__RAW = str
PXD_CLK_DIV = int
PXD_CLK_DIV__TYPE = str
PXD_CLK_DIV__RAW = str
PXD_CLK_DIV_BITS = int
PXD_CLK_DIV_BITS__TYPE = str
PXD_CLK_DIV_BITS__RAW = str
SENS_PHASE_WIDTH = int
SENS_PHASE_WIDTH__TYPE = str
SENS_PHASE_WIDTH__RAW = str
SENS_BANDWIDTH = str
SENS_BANDWIDTH__TYPE = str
SENS_BANDWIDTH__RAW = str
CLKIN_PERIOD_SENSOR = float
CLKIN_PERIOD_SENSOR__TYPE = str
CLKIN_PERIOD_SENSOR__RAW = str
CLKFBOUT_MULT_SENSOR = int
CLKFBOUT_MULT_SENSOR__TYPE = str
CLKFBOUT_MULT_SENSOR__RAW = str
CLKFBOUT_PHASE_SENSOR = float
CLKFBOUT_PHASE_SENSOR__TYPE = str
CLKFBOUT_PHASE_SENSOR__RAW = str
IPCLK_PHASE = float
IPCLK_PHASE__TYPE = str
IPCLK_PHASE__RAW = str
IPCLK2X_PHASE = float
IPCLK2X_PHASE__TYPE = str
IPCLK2X_PHASE__RAW = str
PXD_IOSTANDARD = str
PXD_IOSTANDARD__TYPE = str
PXD_IOSTANDARD__RAW = str
SENSI2C_IOSTANDARD = str
SENSI2C_IOSTANDARD__TYPE = str
SENSI2C_IOSTANDARD__RAW = str
HISPI_UNTUNED_SPLIT = str
HISPI_UNTUNED_SPLIT__TYPE = str
HISPI_UNTUNED_SPLIT__RAW = str
HISPI_DIFF_TERM = str
HISPI_DIFF_TERM__TYPE = str
HISPI_DIFF_TERM__RAW = str
HISPI_IOSTANDARD = str
HISPI_IOSTANDARD__TYPE = str
HISPI_IOSTANDARD__RAW = str
BUF_IPCLK_SENS0 = str
BUF_IPCLK_SENS0__TYPE = str
BUF_IPCLK_SENS0__RAW = str
BUF_IPCLK2X_SENS0 = str
BUF_IPCLK2X_SENS0__TYPE = str
BUF_IPCLK2X_SENS0__RAW = str
BUF_IPCLK_SENS1 = str
BUF_IPCLK_SENS1__TYPE = str
BUF_IPCLK_SENS1__RAW = str
BUF_IPCLK2X_SENS1 = str
BUF_IPCLK2X_SENS1__TYPE = str
BUF_IPCLK2X_SENS1__RAW = str
BUF_IPCLK_SENS2 = str
BUF_IPCLK_SENS2__TYPE = str
BUF_IPCLK_SENS2__RAW = str
BUF_IPCLK2X_SENS2 = str
BUF_IPCLK2X_SENS2__TYPE = str
BUF_IPCLK2X_SENS2__RAW = str
BUF_IPCLK_SENS3 = str
BUF_IPCLK_SENS3__TYPE = str
BUF_IPCLK_SENS3__RAW = str
BUF_IPCLK2X_SENS3 = str
BUF_IPCLK2X_SENS3__TYPE = str
BUF_IPCLK2X_SENS3__RAW = str
SENS_DIVCLK_DIVIDE = int
SENS_DIVCLK_DIVIDE__TYPE = str
SENS_DIVCLK_DIVIDE__RAW = str
SENS_REF_JITTER1 = float
SENS_REF_JITTER1__TYPE = str
SENS_REF_JITTER1__RAW = str
SENS_REF_JITTER2 = float
SENS_REF_JITTER2__TYPE = str
SENS_REF_JITTER2__RAW = str
SENS_SS_EN = str
SENS_SS_EN__TYPE = str
SENS_SS_EN__RAW = str
SENS_SS_MODE = str
SENS_SS_MODE__TYPE = str
SENS_SS_MODE__RAW = str
SENS_SS_MOD_PERIOD = int
SENS_SS_MOD_PERIOD__TYPE = str
SENS_SS_MOD_PERIOD__RAW = str
HISPI_MSB_FIRST = int
HISPI_MSB_FIRST__TYPE = str
HISPI_MSB_FIRST__RAW = str
HISPI_NUMLANES = int
HISPI_NUMLANES__TYPE = str
HISPI_NUMLANES__RAW = str
HISPI_DELAY_CLK0 = str
HISPI_DELAY_CLK0__TYPE = str
HISPI_DELAY_CLK0__RAW = str
HISPI_DELAY_CLK1 = str
HISPI_DELAY_CLK1__TYPE = str
HISPI_DELAY_CLK1__RAW = str
HISPI_DELAY_CLK2 = str
HISPI_DELAY_CLK2__TYPE = str
HISPI_DELAY_CLK2__RAW = str
HISPI_DELAY_CLK3 = str
HISPI_DELAY_CLK3__TYPE = str
HISPI_DELAY_CLK3__RAW = str
HISPI_MMCM0 = str
HISPI_MMCM0__TYPE = str
HISPI_MMCM0__RAW = str
HISPI_MMCM1 = str
HISPI_MMCM1__TYPE = str
HISPI_MMCM1__RAW = str
HISPI_MMCM2 = str
HISPI_MMCM2__TYPE = str
HISPI_MMCM2__RAW = str
HISPI_MMCM3 = str
HISPI_MMCM3__TYPE = str
HISPI_MMCM3__RAW = str
HISPI_KEEP_IRST = int
HISPI_KEEP_IRST__TYPE = str
HISPI_KEEP_IRST__RAW = str
HISPI_WAIT_ALL_LANES = int
HISPI_WAIT_ALL_LANES__TYPE = str
HISPI_WAIT_ALL_LANES__RAW = str
HISPI_FIFO_DEPTH = int
HISPI_FIFO_DEPTH__TYPE = str
HISPI_FIFO_DEPTH__RAW = str
HISPI_FIFO_START = int
HISPI_FIFO_START__TYPE = str
HISPI_FIFO_START__RAW = str
HISPI_CAPACITANCE = str
HISPI_CAPACITANCE__TYPE = str
HISPI_CAPACITANCE__RAW = str
HISPI_DQS_BIAS = str
HISPI_DQS_BIAS__TYPE = str
HISPI_DQS_BIAS__RAW = str
HISPI_IBUF_DELAY_VALUE = str
HISPI_IBUF_DELAY_VALUE__TYPE = str
HISPI_IBUF_DELAY_VALUE__RAW = str
HISPI_IBUF_LOW_PWR = str
HISPI_IBUF_LOW_PWR__TYPE = str
HISPI_IBUF_LOW_PWR__RAW = str
HISPI_IFD_DELAY_VALUE = str
HISPI_IFD_DELAY_VALUE__TYPE = str
HISPI_IFD_DELAY_VALUE__RAW = str
CMPRS_NUM_AFI_CHN = int
CMPRS_NUM_AFI_CHN__TYPE = str
CMPRS_NUM_AFI_CHN__RAW = str
CMPRS_GROUP_ADDR = int
CMPRS_GROUP_ADDR__TYPE = str
CMPRS_GROUP_ADDR__RAW = str
CMPRS_BASE_INC = int
CMPRS_BASE_INC__TYPE = str
CMPRS_BASE_INC__RAW = str
CMPRS_AFIMUX_RADDR0 = int
CMPRS_AFIMUX_RADDR0__TYPE = str
CMPRS_AFIMUX_RADDR0__RAW = str
CMPRS_AFIMUX_RADDR1 = int
CMPRS_AFIMUX_RADDR1__TYPE = str
CMPRS_AFIMUX_RADDR1__RAW = str
CMPRS_AFIMUX_MASK = int
CMPRS_AFIMUX_MASK__TYPE = str
CMPRS_AFIMUX_MASK__RAW = str
CMPRS_STATUS_REG_BASE = int
CMPRS_STATUS_REG_BASE__TYPE = str
CMPRS_STATUS_REG_BASE__RAW = str
CMPRS_HIFREQ_REG_BASE = int
CMPRS_HIFREQ_REG_BASE__TYPE = str
CMPRS_HIFREQ_REG_BASE__RAW = str
CMPRS_AFIMUX_REG_ADDR0 = int
CMPRS_AFIMUX_REG_ADDR0__TYPE = str
CMPRS_AFIMUX_REG_ADDR0__RAW = str
CMPRS_AFIMUX_REG_ADDR1 = int
CMPRS_AFIMUX_REG_ADDR1__TYPE = str
CMPRS_AFIMUX_REG_ADDR1__RAW = str
CMPRS_STATUS_REG_INC = int
CMPRS_STATUS_REG_INC__TYPE = str
CMPRS_STATUS_REG_INC__RAW = str
CMPRS_HIFREQ_REG_INC = int
CMPRS_HIFREQ_REG_INC__TYPE = str
CMPRS_HIFREQ_REG_INC__RAW = str
CMPRS_MASK = int
CMPRS_MASK__TYPE = str
CMPRS_MASK__RAW = str
CMPRS_CONTROL_REG = int
CMPRS_CONTROL_REG__TYPE = str
CMPRS_CONTROL_REG__RAW = str
CMPRS_STATUS_CNTRL = int
CMPRS_STATUS_CNTRL__TYPE = str
CMPRS_STATUS_CNTRL__RAW = str
CMPRS_FORMAT = int
CMPRS_FORMAT__TYPE = str
CMPRS_FORMAT__RAW = str
CMPRS_COLOR_SATURATION = int
CMPRS_COLOR_SATURATION__TYPE = str
CMPRS_COLOR_SATURATION__RAW = str
CMPRS_CORING_MODE = int
CMPRS_CORING_MODE__TYPE = str
CMPRS_CORING_MODE__RAW = str
CMPRS_INTERRUPTS = int
CMPRS_INTERRUPTS__TYPE = str
CMPRS_INTERRUPTS__RAW = str
CMPRS_TABLES = int
CMPRS_TABLES__TYPE = str
CMPRS_TABLES__RAW = str
TABLE_QUANTIZATION_INDEX = int
TABLE_QUANTIZATION_INDEX__TYPE = str
TABLE_QUANTIZATION_INDEX__RAW = str
TABLE_CORING_INDEX = int
TABLE_CORING_INDEX__TYPE = str
TABLE_CORING_INDEX__RAW = str
TABLE_FOCUS_INDEX = int
TABLE_FOCUS_INDEX__TYPE = str
TABLE_FOCUS_INDEX__RAW = str
TABLE_HUFFMAN_INDEX = int
TABLE_HUFFMAN_INDEX__TYPE = str
TABLE_HUFFMAN_INDEX__RAW = str
CMPRS_CBIT_RUN = int
CMPRS_CBIT_RUN__TYPE = str
CMPRS_CBIT_RUN__RAW = str
CMPRS_CBIT_RUN_BITS = int
CMPRS_CBIT_RUN_BITS__TYPE = str
CMPRS_CBIT_RUN_BITS__RAW = str
CMPRS_CBIT_QBANK = int
CMPRS_CBIT_QBANK__TYPE = str
CMPRS_CBIT_QBANK__RAW = str
CMPRS_CBIT_QBANK_BITS = int
CMPRS_CBIT_QBANK_BITS__TYPE = str
CMPRS_CBIT_QBANK_BITS__RAW = str
CMPRS_CBIT_DCSUB = int
CMPRS_CBIT_DCSUB__TYPE = str
CMPRS_CBIT_DCSUB__RAW = str
CMPRS_CBIT_DCSUB_BITS = int
CMPRS_CBIT_DCSUB_BITS__TYPE = str
CMPRS_CBIT_DCSUB_BITS__RAW = str
CMPRS_CBIT_CMODE = int
CMPRS_CBIT_CMODE__TYPE = str
CMPRS_CBIT_CMODE__RAW = str
CMPRS_CBIT_CMODE_BITS = int
CMPRS_CBIT_CMODE_BITS__TYPE = str
CMPRS_CBIT_CMODE_BITS__RAW = str
CMPRS_CBIT_FRAMES = int
CMPRS_CBIT_FRAMES__TYPE = str
CMPRS_CBIT_FRAMES__RAW = str
CMPRS_CBIT_FRAMES_BITS = int
CMPRS_CBIT_FRAMES_BITS__TYPE = str
CMPRS_CBIT_FRAMES_BITS__RAW = str
CMPRS_CBIT_BE16 = int
CMPRS_CBIT_BE16__TYPE = str
CMPRS_CBIT_BE16__RAW = str
CMPRS_CBIT_BE16_BITS = int
CMPRS_CBIT_BE16_BITS__TYPE = str
CMPRS_CBIT_BE16_BITS__RAW = str
CMPRS_CBIT_BAYER = int
CMPRS_CBIT_BAYER__TYPE = str
CMPRS_CBIT_BAYER__RAW = str
CMPRS_CBIT_BAYER_BITS = int
CMPRS_CBIT_BAYER_BITS__TYPE = str
CMPRS_CBIT_BAYER_BITS__RAW = str
CMPRS_CBIT_FOCUS = int
CMPRS_CBIT_FOCUS__TYPE = str
CMPRS_CBIT_FOCUS__RAW = str
CMPRS_CBIT_FOCUS_BITS = int
CMPRS_CBIT_FOCUS_BITS__TYPE = str
CMPRS_CBIT_FOCUS_BITS__RAW = str
CMPRS_CBIT_ROWS_LSB = int
CMPRS_CBIT_ROWS_LSB__TYPE = str
CMPRS_CBIT_ROWS_LSB__RAW = str
CMPRS_CBIT_ROWS_LSB_BITS = int
CMPRS_CBIT_ROWS_LSB_BITS__TYPE = str
CMPRS_CBIT_ROWS_LSB_BITS__RAW = str
CMPRS_CBIT_RUN_RST = int
CMPRS_CBIT_RUN_RST__TYPE = str
CMPRS_CBIT_RUN_RST__RAW = str
CMPRS_CBIT_RUN_STANDALONE = int
CMPRS_CBIT_RUN_STANDALONE__TYPE = str
CMPRS_CBIT_RUN_STANDALONE__RAW = str
CMPRS_CBIT_RUN_ENABLE = int
CMPRS_CBIT_RUN_ENABLE__TYPE = str
CMPRS_CBIT_RUN_ENABLE__RAW = str
CMPRS_CBIT_CMODE_JPEG18 = int
CMPRS_CBIT_CMODE_JPEG18__TYPE = str
CMPRS_CBIT_CMODE_JPEG18__RAW = str
CMPRS_CBIT_CMODE_MONO6 = int
CMPRS_CBIT_CMODE_MONO6__TYPE = str
CMPRS_CBIT_CMODE_MONO6__RAW = str
CMPRS_CBIT_CMODE_JP46 = int
CMPRS_CBIT_CMODE_JP46__TYPE = str
CMPRS_CBIT_CMODE_JP46__RAW = str
CMPRS_CBIT_CMODE_JP46DC = int
CMPRS_CBIT_CMODE_JP46DC__TYPE = str
CMPRS_CBIT_CMODE_JP46DC__RAW = str
CMPRS_CBIT_CMODE_JPEG20 = int
CMPRS_CBIT_CMODE_JPEG20__TYPE = str
CMPRS_CBIT_CMODE_JPEG20__RAW = str
CMPRS_CBIT_CMODE_JP4 = int
CMPRS_CBIT_CMODE_JP4__TYPE = str
CMPRS_CBIT_CMODE_JP4__RAW = str
CMPRS_CBIT_CMODE_JP4DC = int
CMPRS_CBIT_CMODE_JP4DC__TYPE = str
CMPRS_CBIT_CMODE_JP4DC__RAW = str
CMPRS_CBIT_CMODE_JP4DIFF = int
CMPRS_CBIT_CMODE_JP4DIFF__TYPE = str
CMPRS_CBIT_CMODE_JP4DIFF__RAW = str
CMPRS_CBIT_CMODE_JP4DIFFHDR = int
CMPRS_CBIT_CMODE_JP4DIFFHDR__TYPE = str
CMPRS_CBIT_CMODE_JP4DIFFHDR__RAW = str
CMPRS_CBIT_CMODE_JP4DIFFDIV2 = int
CMPRS_CBIT_CMODE_JP4DIFFDIV2__TYPE = str
CMPRS_CBIT_CMODE_JP4DIFFDIV2__RAW = str
CMPRS_CBIT_CMODE_JP4DIFFHDRDIV2 = int
CMPRS_CBIT_CMODE_JP4DIFFHDRDIV2__TYPE = str
CMPRS_CBIT_CMODE_JP4DIFFHDRDIV2__RAW = str
CMPRS_CBIT_CMODE_MONO1 = int
CMPRS_CBIT_CMODE_MONO1__TYPE = str
CMPRS_CBIT_CMODE_MONO1__RAW = str
CMPRS_CBIT_CMODE_MONO4 = int
CMPRS_CBIT_CMODE_MONO4__TYPE = str
CMPRS_CBIT_CMODE_MONO4__RAW = str
CMPRS_CBIT_CMODE_RAW = int
CMPRS_CBIT_CMODE_RAW__TYPE = str
CMPRS_CBIT_CMODE_RAW__RAW = str
CMPRS_CBIT_FRAMES_SINGLE = int
CMPRS_CBIT_FRAMES_SINGLE__TYPE = str
CMPRS_CBIT_FRAMES_SINGLE__RAW = str
CMPRS_COLOR18 = int
CMPRS_COLOR18__TYPE = str
CMPRS_COLOR18__RAW = str
CMPRS_COLOR20 = int
CMPRS_COLOR20__TYPE = str
CMPRS_COLOR20__RAW = str
CMPRS_MONO16 = int
CMPRS_MONO16__TYPE = str
CMPRS_MONO16__RAW = str
CMPRS_JP4 = int
CMPRS_JP4__TYPE = str
CMPRS_JP4__RAW = str
CMPRS_JP4DIFF = int
CMPRS_JP4DIFF__TYPE = str
CMPRS_JP4DIFF__RAW = str
CMPRS_RAW = int
CMPRS_RAW__TYPE = str
CMPRS_RAW__RAW = str
CMPRS_MONO8 = int
CMPRS_MONO8__TYPE = str
CMPRS_MONO8__RAW = str
CMPRS_FRMT_MBCM1 = int
CMPRS_FRMT_MBCM1__TYPE = str
CMPRS_FRMT_MBCM1__RAW = str
CMPRS_FRMT_MBCM1_BITS = int
CMPRS_FRMT_MBCM1_BITS__TYPE = str
CMPRS_FRMT_MBCM1_BITS__RAW = str
CMPRS_FRMT_MBRM1 = int
CMPRS_FRMT_MBRM1__TYPE = str
CMPRS_FRMT_MBRM1__RAW = str
CMPRS_FRMT_MBRM1_BITS = int
CMPRS_FRMT_MBRM1_BITS__TYPE = str
CMPRS_FRMT_MBRM1_BITS__RAW = str
CMPRS_FRMT_LMARG = int
CMPRS_FRMT_LMARG__TYPE = str
CMPRS_FRMT_LMARG__RAW = str
CMPRS_FRMT_LMARG_BITS = int
CMPRS_FRMT_LMARG_BITS__TYPE = str
CMPRS_FRMT_LMARG_BITS__RAW = str
CMPRS_CSAT_CB = int
CMPRS_CSAT_CB__TYPE = str
CMPRS_CSAT_CB__RAW = str
CMPRS_CSAT_CB_BITS = int
CMPRS_CSAT_CB_BITS__TYPE = str
CMPRS_CSAT_CB_BITS__RAW = str
CMPRS_CSAT_CR = int
CMPRS_CSAT_CR__TYPE = str
CMPRS_CSAT_CR__RAW = str
CMPRS_CSAT_CR_BITS = int
CMPRS_CSAT_CR_BITS__TYPE = str
CMPRS_CSAT_CR_BITS__RAW = str
CMPRS_CORING_BITS = int
CMPRS_CORING_BITS__TYPE = str
CMPRS_CORING_BITS__RAW = str
CMPRS_TIMEOUT_BITS = int
CMPRS_TIMEOUT_BITS__TYPE = str
CMPRS_TIMEOUT_BITS__RAW = str
CMPRS_TIMEOUT = int
CMPRS_TIMEOUT__TYPE = str
CMPRS_TIMEOUT__RAW = str
CMPRS_AFIMUX_EN = int
CMPRS_AFIMUX_EN__TYPE = str
CMPRS_AFIMUX_EN__RAW = str
CMPRS_AFIMUX_RST = int
CMPRS_AFIMUX_RST__TYPE = str
CMPRS_AFIMUX_RST__RAW = str
CMPRS_AFIMUX_MODE = int
CMPRS_AFIMUX_MODE__TYPE = str
CMPRS_AFIMUX_MODE__RAW = str
CMPRS_AFIMUX_STATUS_CNTRL = int
CMPRS_AFIMUX_STATUS_CNTRL__TYPE = str
CMPRS_AFIMUX_STATUS_CNTRL__RAW = str
CMPRS_AFIMUX_SA_LEN = int
CMPRS_AFIMUX_SA_LEN__TYPE = str
CMPRS_AFIMUX_SA_LEN__RAW = str
CMPRS_AFIMUX_WIDTH = int
CMPRS_AFIMUX_WIDTH__TYPE = str
CMPRS_AFIMUX_WIDTH__RAW = str
CMPRS_AFIMUX_CYCBITS = int
CMPRS_AFIMUX_CYCBITS__TYPE = str
CMPRS_AFIMUX_CYCBITS__RAW = str
AFI_MUX_BUF_LATENCY = int
AFI_MUX_BUF_LATENCY__TYPE = str
AFI_MUX_BUF_LATENCY__RAW = str
GPIO_DRIVE = int
GPIO_DRIVE__TYPE = str
GPIO_DRIVE__RAW = str
GPIO_ADDR = int
GPIO_ADDR__TYPE = str
GPIO_ADDR__RAW = str
GPIO_MASK = int
GPIO_MASK__TYPE = str
GPIO_MASK__RAW = str
GPIO_STATUS_REG_ADDR = int
GPIO_STATUS_REG_ADDR__TYPE = str
GPIO_STATUS_REG_ADDR__RAW = str
GPIO_IBUF_LOW_PWR = str
GPIO_IBUF_LOW_PWR__TYPE = str
GPIO_IBUF_LOW_PWR__RAW = str
GPIO_IOSTANDARD = str
GPIO_IOSTANDARD__TYPE = str
GPIO_IOSTANDARD__RAW = str
GPIO_SLEW = str
GPIO_SLEW__TYPE = str
GPIO_SLEW__RAW = str
GPIO_SET_PINS = int
GPIO_SET_PINS__TYPE = str
GPIO_SET_PINS__RAW = str
GPIO_SET_STATUS = int
GPIO_SET_STATUS__TYPE = str
GPIO_SET_STATUS__RAW = str
GPIO_N = int
GPIO_N__TYPE = str
GPIO_N__RAW = str
GPIO_PORTEN = int
GPIO_PORTEN__TYPE = str
GPIO_PORTEN__RAW = str
RTC_ADDR = int
RTC_ADDR__TYPE = str
RTC_ADDR__RAW = str
CAMSYNC_ADDR = int
CAMSYNC_ADDR__TYPE = str
CAMSYNC_ADDR__RAW = str
RTC_STATUS_REG_ADDR = int
RTC_STATUS_REG_ADDR__TYPE = str
RTC_STATUS_REG_ADDR__RAW = str
RTC_SEC_USEC_ADDR = int
RTC_SEC_USEC_ADDR__TYPE = str
RTC_SEC_USEC_ADDR__RAW = str
RTC_MASK = int
RTC_MASK__TYPE = str
RTC_MASK__RAW = str
CAMSYNC_MASK = int
CAMSYNC_MASK__TYPE = str
CAMSYNC_MASK__RAW = str
CAMSYNC_MODE = int
CAMSYNC_MODE__TYPE = str
CAMSYNC_MODE__RAW = str
CAMSYNC_TRIG_SRC = int
CAMSYNC_TRIG_SRC__TYPE = str
CAMSYNC_TRIG_SRC__RAW = str
CAMSYNC_TRIG_DST = int
CAMSYNC_TRIG_DST__TYPE = str
CAMSYNC_TRIG_DST__RAW = str
CAMSYNC_TRIG_PERIOD = int
CAMSYNC_TRIG_PERIOD__TYPE = str
CAMSYNC_TRIG_PERIOD__RAW = str
CAMSYNC_TRIG_DELAY0 = int
CAMSYNC_TRIG_DELAY0__TYPE = str
CAMSYNC_TRIG_DELAY0__RAW = str
CAMSYNC_TRIG_DELAY1 = int
CAMSYNC_TRIG_DELAY1__TYPE = str
CAMSYNC_TRIG_DELAY1__RAW = str
CAMSYNC_TRIG_DELAY2 = int
CAMSYNC_TRIG_DELAY2__TYPE = str
CAMSYNC_TRIG_DELAY2__RAW = str
CAMSYNC_TRIG_DELAY3 = int
CAMSYNC_TRIG_DELAY3__TYPE = str
CAMSYNC_TRIG_DELAY3__RAW = str
CAMSYNC_EN_BIT = int
CAMSYNC_EN_BIT__TYPE = str
CAMSYNC_EN_BIT__RAW = str
CAMSYNC_SNDEN_BIT = int
CAMSYNC_SNDEN_BIT__TYPE = str
CAMSYNC_SNDEN_BIT__RAW = str
CAMSYNC_EXTERNAL_BIT = int
CAMSYNC_EXTERNAL_BIT__TYPE = str
CAMSYNC_EXTERNAL_BIT__RAW = str
CAMSYNC_TRIGGERED_BIT = int
CAMSYNC_TRIGGERED_BIT__TYPE = str
CAMSYNC_TRIGGERED_BIT__RAW = str
CAMSYNC_MASTER_BIT = int
CAMSYNC_MASTER_BIT__TYPE = str
CAMSYNC_MASTER_BIT__RAW = str
CAMSYNC_CHN_EN_BIT = int
CAMSYNC_CHN_EN_BIT__TYPE = str
CAMSYNC_CHN_EN_BIT__RAW = str
CAMSYNC_PRE_MAGIC = int
CAMSYNC_PRE_MAGIC__TYPE = str
CAMSYNC_PRE_MAGIC__RAW = str
CAMSYNC_POST_MAGIC = int
CAMSYNC_POST_MAGIC__TYPE = str
CAMSYNC_POST_MAGIC__RAW = str
CAMSYNC_GPIO_EXT_IN = int
CAMSYNC_GPIO_EXT_IN__TYPE = str
CAMSYNC_GPIO_EXT_IN__RAW = str
CAMSYNC_GPIO_INT_IN = int
CAMSYNC_GPIO_INT_IN__TYPE = str
CAMSYNC_GPIO_INT_IN__RAW = str
CAMSYNC_GPIO_EXT_OUT = int
CAMSYNC_GPIO_EXT_OUT__TYPE = str
CAMSYNC_GPIO_EXT_OUT__RAW = str
CAMSYNC_GPIO_INT_OUT = int
CAMSYNC_GPIO_INT_OUT__TYPE = str
CAMSYNC_GPIO_INT_OUT__RAW = str
RTC_MHZ = int
RTC_MHZ__TYPE = str
RTC_MHZ__RAW = str
RTC_BITC_PREDIV = int
RTC_BITC_PREDIV__TYPE = str
RTC_BITC_PREDIV__RAW = str
RTC_SET_USEC = int
RTC_SET_USEC__TYPE = str
RTC_SET_USEC__RAW = str
RTC_SET_SEC = int
RTC_SET_SEC__TYPE = str
RTC_SET_SEC__RAW = str
RTC_SET_CORR = int
RTC_SET_CORR__TYPE = str
RTC_SET_CORR__RAW = str
RTC_SET_STATUS = int
RTC_SET_STATUS__TYPE = str
RTC_SET_STATUS__RAW = str
CMDFRAMESEQ_ADDR_BASE = int
CMDFRAMESEQ_ADDR_BASE__TYPE = str
CMDFRAMESEQ_ADDR_BASE__RAW = str
CMDFRAMESEQ_ADDR_INC = int
CMDFRAMESEQ_ADDR_INC__TYPE = str
CMDFRAMESEQ_ADDR_INC__RAW = str
CMDFRAMESEQ_MASK = int
CMDFRAMESEQ_MASK__TYPE = str
CMDFRAMESEQ_MASK__RAW = str
CMDFRAMESEQ_DEPTH = int
CMDFRAMESEQ_DEPTH__TYPE = str
CMDFRAMESEQ_DEPTH__RAW = str
CMDFRAMESEQ_ABS = int
CMDFRAMESEQ_ABS__TYPE = str
CMDFRAMESEQ_ABS__RAW = str
CMDFRAMESEQ_REL = int
CMDFRAMESEQ_REL__TYPE = str
CMDFRAMESEQ_REL__RAW = str
CMDFRAMESEQ_CTRL = int
CMDFRAMESEQ_CTRL__TYPE = str
CMDFRAMESEQ_CTRL__RAW = str
CMDFRAMESEQ_RST_BIT = int
CMDFRAMESEQ_RST_BIT__TYPE = str
CMDFRAMESEQ_RST_BIT__RAW = str
CMDFRAMESEQ_RUN_BIT = int
CMDFRAMESEQ_RUN_BIT__TYPE = str
CMDFRAMESEQ_RUN_BIT__RAW = str
CMDFRAMESEQ_IRQ_BIT = int
CMDFRAMESEQ_IRQ_BIT__TYPE = str
CMDFRAMESEQ_IRQ_BIT__RAW = str
CMDSEQMUX_ADDR = int
CMDSEQMUX_ADDR__TYPE = str
CMDSEQMUX_ADDR__RAW = str
CMDSEQMUX_MASK = int
CMDSEQMUX_MASK__TYPE = str
CMDSEQMUX_MASK__RAW = str
CMDSEQMUX_STATUS = int
CMDSEQMUX_STATUS__TYPE = str
CMDSEQMUX_STATUS__RAW = str
LOGGER_ADDR = int
LOGGER_ADDR__TYPE = str
LOGGER_ADDR__RAW = str
LOGGER_STATUS = int
LOGGER_STATUS__TYPE = str
LOGGER_STATUS__RAW = str
LOGGER_STATUS_REG_ADDR = int
LOGGER_STATUS_REG_ADDR__TYPE = str
LOGGER_STATUS_REG_ADDR__RAW = str
LOGGER_MASK = int
LOGGER_MASK__TYPE = str
LOGGER_MASK__RAW = str
LOGGER_STATUS_MASK = int
LOGGER_STATUS_MASK__TYPE = str
LOGGER_STATUS_MASK__RAW = str
LOGGER_PAGE_IMU = int
LOGGER_PAGE_IMU__TYPE = str
LOGGER_PAGE_IMU__RAW = str
LOGGER_PAGE_GPS = int
LOGGER_PAGE_GPS__TYPE = str
LOGGER_PAGE_GPS__RAW = str
LOGGER_PAGE_MSG = int
LOGGER_PAGE_MSG__TYPE = str
LOGGER_PAGE_MSG__RAW = str
LOGGER_PERIOD = int
LOGGER_PERIOD__TYPE = str
LOGGER_PERIOD__RAW = str
LOGGER_BIT_DURATION = int
LOGGER_BIT_DURATION__TYPE = str
LOGGER_BIT_DURATION__RAW = str
LOGGER_BIT_HALF_PERIOD = int
LOGGER_BIT_HALF_PERIOD__TYPE = str
LOGGER_BIT_HALF_PERIOD__RAW = str
LOGGER_CONFIG = int
LOGGER_CONFIG__TYPE = str
LOGGER_CONFIG__RAW = str
LOGGER_CONF_IMU = int
LOGGER_CONF_IMU__TYPE = str
LOGGER_CONF_IMU__RAW = str
LOGGER_CONF_IMU_BITS = int
LOGGER_CONF_IMU_BITS__TYPE = str
LOGGER_CONF_IMU_BITS__RAW = str
LOGGER_CONF_GPS = int
LOGGER_CONF_GPS__TYPE = str
LOGGER_CONF_GPS__RAW = str
LOGGER_CONF_GPS_BITS = int
LOGGER_CONF_GPS_BITS__TYPE = str
LOGGER_CONF_GPS_BITS__RAW = str
LOGGER_CONF_MSG = int
LOGGER_CONF_MSG__TYPE = str
LOGGER_CONF_MSG__RAW = str
LOGGER_CONF_MSG_BITS = int
LOGGER_CONF_MSG_BITS__TYPE = str
LOGGER_CONF_MSG_BITS__RAW = str
LOGGER_CONF_SYN = int
LOGGER_CONF_SYN__TYPE = str
LOGGER_CONF_SYN__RAW = str
LOGGER_CONF_SYN_BITS = int
LOGGER_CONF_SYN_BITS__TYPE = str
LOGGER_CONF_SYN_BITS__RAW = str
LOGGER_CONF_EN = int
LOGGER_CONF_EN__TYPE = str
LOGGER_CONF_EN__RAW = str
LOGGER_CONF_EN_BITS = int
LOGGER_CONF_EN_BITS__TYPE = str
LOGGER_CONF_EN_BITS__RAW = str
LOGGER_CONF_DBG = int
LOGGER_CONF_DBG__TYPE = str
LOGGER_CONF_DBG__RAW = str
LOGGER_CONF_DBG_BITS = int
LOGGER_CONF_DBG_BITS__TYPE = str
LOGGER_CONF_DBG_BITS__RAW = str
MULT_SAXI_HALF_BRAM_IN = int
MULT_SAXI_HALF_BRAM_IN__TYPE = str
MULT_SAXI_HALF_BRAM_IN__RAW = str
MULT_SAXI_WLOG = int
MULT_SAXI_WLOG__TYPE = str
MULT_SAXI_WLOG__RAW = str
MULT_SAXI_ADDR = int
MULT_SAXI_ADDR__TYPE = str
MULT_SAXI_ADDR__RAW = str
MULT_SAXI_IRQLEN_ADDR = int
MULT_SAXI_IRQLEN_ADDR__TYPE = str
MULT_SAXI_IRQLEN_ADDR__RAW = str
MULT_SAXI_CNTRL_ADDR = int
MULT_SAXI_CNTRL_ADDR__TYPE = str
MULT_SAXI_CNTRL_ADDR__RAW = str
MULT_SAXI_CNTRL_MODE = int
MULT_SAXI_CNTRL_MODE__TYPE = str
MULT_SAXI_CNTRL_MODE__RAW = str
MULT_SAXI_CNTRL_STATUS = int
MULT_SAXI_CNTRL_STATUS__TYPE = str
MULT_SAXI_CNTRL_STATUS__RAW = str
MULT_SAXI_CNTRL_IRQ = int
MULT_SAXI_CNTRL_IRQ__TYPE = str
MULT_SAXI_CNTRL_IRQ__RAW = str
MULT_SAXI_POINTERS_REG = int
MULT_SAXI_POINTERS_REG__TYPE = str
MULT_SAXI_POINTERS_REG__RAW = str
MULT_SAXI_STATUS_REG = int
MULT_SAXI_STATUS_REG__TYPE = str
MULT_SAXI_STATUS_REG__RAW = str
MULT_SAXI_HALF_BRAM = int
MULT_SAXI_HALF_BRAM__TYPE = str
MULT_SAXI_HALF_BRAM__RAW = str
MULT_SAXI_BSLOG0 = int
MULT_SAXI_BSLOG0__TYPE = str
MULT_SAXI_BSLOG0__RAW = str
MULT_SAXI_BSLOG1 = int
MULT_SAXI_BSLOG1__TYPE = str
MULT_SAXI_BSLOG1__RAW = str
MULT_SAXI_BSLOG2 = int
MULT_SAXI_BSLOG2__TYPE = str
MULT_SAXI_BSLOG2__RAW = str
MULT_SAXI_BSLOG3 = int
MULT_SAXI_BSLOG3__TYPE = str
MULT_SAXI_BSLOG3__RAW = str
MULT_SAXI_MASK = int
MULT_SAXI_MASK__TYPE = str
MULT_SAXI_MASK__RAW = str
MULT_SAXI_IRQLEN_MASK = int
MULT_SAXI_IRQLEN_MASK__TYPE = str
MULT_SAXI_IRQLEN_MASK__RAW = str
MULT_SAXI_CNTRL_MASK = int
MULT_SAXI_CNTRL_MASK__TYPE = str
MULT_SAXI_CNTRL_MASK__RAW = str
MULT_SAXI_AWCACHE = int
MULT_SAXI_AWCACHE__TYPE = str
MULT_SAXI_AWCACHE__RAW = str
MULT_SAXI_ADV_WR = int
MULT_SAXI_ADV_WR__TYPE = str
MULT_SAXI_ADV_WR__RAW = str
MULT_SAXI_ADV_RD = int
MULT_SAXI_ADV_RD__TYPE = str
MULT_SAXI_ADV_RD__RAW = str
CLK_ADDR = int
CLK_ADDR__TYPE = str
CLK_ADDR__RAW = str
CLK_MASK = int
CLK_MASK__TYPE = str
CLK_MASK__RAW = str
CLK_STATUS_REG_ADDR = int
CLK_STATUS_REG_ADDR__TYPE = str
CLK_STATUS_REG_ADDR__RAW = str
CLK_CNTRL = int
CLK_CNTRL__TYPE = str
CLK_CNTRL__RAW = str
CLK_STATUS = int
CLK_STATUS__TYPE = str
CLK_STATUS__RAW = str
DEBUG_ADDR = int
DEBUG_ADDR__TYPE = str
DEBUG_ADDR__RAW = str
DEBUG_MASK = int
DEBUG_MASK__TYPE = str
DEBUG_MASK__RAW = str
DEBUG_STATUS_REG_ADDR = int
DEBUG_STATUS_REG_ADDR__TYPE = str
DEBUG_STATUS_REG_ADDR__RAW = str
DEBUG_READ_REG_ADDR = int
DEBUG_READ_REG_ADDR__TYPE = str
DEBUG_READ_REG_ADDR__RAW = str
DEBUG_SHIFT_DATA = int
DEBUG_SHIFT_DATA__TYPE = str
DEBUG_SHIFT_DATA__RAW = str
DEBUG_LOAD = int
DEBUG_LOAD__TYPE = str
DEBUG_LOAD__RAW = str
DEBUG_SET_STATUS = int
DEBUG_SET_STATUS__TYPE = str
DEBUG_SET_STATUS__RAW = str
DEBUG_CMD_LATENCY = int
DEBUG_CMD_LATENCY__TYPE = str
DEBUG_CMD_LATENCY__RAW = str
MULTICLK_IN_PERIOD = int
MULTICLK_IN_PERIOD__TYPE = str
MULTICLK_IN_PERIOD__RAW = str
MULTICLK_DIVCLK = int
MULTICLK_DIVCLK__TYPE = str
MULTICLK_DIVCLK__RAW = str
MULTICLK_MULT = int
MULTICLK_MULT__TYPE = str
MULTICLK_MULT__RAW = str
MULTICLK_DIV_DLYREF = int
MULTICLK_DIV_DLYREF__TYPE = str
MULTICLK_DIV_DLYREF__RAW = str
MULTICLK_DIV_AXIHP = int
MULTICLK_DIV_AXIHP__TYPE = str
MULTICLK_DIV_AXIHP__RAW = str
MULTICLK_DIV_XCLK = int
MULTICLK_DIV_XCLK__TYPE = str
MULTICLK_DIV_XCLK__RAW = str
MULTICLK_DIV_XCLK2X = int
MULTICLK_DIV_XCLK2X__TYPE = str
MULTICLK_DIV_XCLK2X__RAW = str
MULTICLK_DIV_SYNC = int
MULTICLK_DIV_SYNC__TYPE = str
MULTICLK_DIV_SYNC__RAW = str
MULTICLK_PHASE_FB = float
MULTICLK_PHASE_FB__TYPE = str
MULTICLK_PHASE_FB__RAW = str
MULTICLK_PHASE_DLYREF = float
MULTICLK_PHASE_DLYREF__TYPE = str
MULTICLK_PHASE_DLYREF__RAW = str
MULTICLK_BUF_DLYREF = str
MULTICLK_BUF_DLYREF__TYPE = str
MULTICLK_BUF_DLYREF__RAW = str
MULTICLK_PHASE_AXIHP = float
MULTICLK_PHASE_AXIHP__TYPE = str
MULTICLK_PHASE_AXIHP__RAW = str
MULTICLK_BUF_AXIHP = str
MULTICLK_BUF_AXIHP__TYPE = str
MULTICLK_BUF_AXIHP__RAW = str
MULTICLK_PHASE_XCLK = float
MULTICLK_PHASE_XCLK__TYPE = str
MULTICLK_PHASE_XCLK__RAW = str
MULTICLK_BUF_XCLK = str
MULTICLK_BUF_XCLK__TYPE = str
MULTICLK_BUF_XCLK__RAW = str
MULTICLK_PHASE_SYNC = float
MULTICLK_PHASE_SYNC__TYPE = str
MULTICLK_PHASE_SYNC__RAW = str
MULTICLK_BUF_SYNC = str
MULTICLK_BUF_SYNC__TYPE = str
MULTICLK_BUF_SYNC__RAW = str
CLKIN_PERIOD_PCLK = int
CLKIN_PERIOD_PCLK__TYPE = str
CLKIN_PERIOD_PCLK__RAW = str
DIVCLK_DIVIDE_PCLK = int
DIVCLK_DIVIDE_PCLK__TYPE = str
DIVCLK_DIVIDE_PCLK__RAW = str
CLKFBOUT_MULT_PCLK = int
CLKFBOUT_MULT_PCLK__TYPE = str
CLKFBOUT_MULT_PCLK__RAW = str
CLKOUT_DIV_PCLK = int
CLKOUT_DIV_PCLK__TYPE = str
CLKOUT_DIV_PCLK__RAW = str
CLKOUT_DIV_PCLK2X = int
CLKOUT_DIV_PCLK2X__TYPE = str
CLKOUT_DIV_PCLK2X__RAW = str
PHASE_CLK2X_PCLK = float
PHASE_CLK2X_PCLK__TYPE = str
PHASE_CLK2X_PCLK__RAW = str
BUF_CLK1X_PCLK = str
BUF_CLK1X_PCLK__TYPE = str
BUF_CLK1X_PCLK__RAW = str
BUF_CLK1X_PCLK2X = str
BUF_CLK1X_PCLK2X__TYPE = str
BUF_CLK1X_PCLK2X__RAW = str
MEMCLK_CAPACITANCE = str
MEMCLK_CAPACITANCE__TYPE = str
MEMCLK_CAPACITANCE__RAW = str
MEMCLK_IBUF_LOW_PWR = str
MEMCLK_IBUF_LOW_PWR__TYPE = str
MEMCLK_IBUF_LOW_PWR__RAW = str
MEMCLK_IOSTANDARD = str
MEMCLK_IOSTANDARD__TYPE = str
MEMCLK_IOSTANDARD__RAW = str
FFCLK0_CAPACITANCE = str
FFCLK0_CAPACITANCE__TYPE = str
FFCLK0_CAPACITANCE__RAW = str
FFCLK0_DIFF_TERM = str
FFCLK0_DIFF_TERM__TYPE = str
FFCLK0_DIFF_TERM__RAW = str
FFCLK0_IBUF_LOW_PWR = str
FFCLK0_IBUF_LOW_PWR__TYPE = str
FFCLK0_IBUF_LOW_PWR__RAW = str
FFCLK0_IOSTANDARD = str
FFCLK0_IOSTANDARD__TYPE = str
FFCLK0_IOSTANDARD__RAW = str
FFCLK1_CAPACITANCE = str
FFCLK1_CAPACITANCE__TYPE = str
FFCLK1_CAPACITANCE__RAW = str
FFCLK1_DIFF_TERM = str
FFCLK1_DIFF_TERM__TYPE = str
FFCLK1_DIFF_TERM__RAW = str
FFCLK1_IBUF_LOW_PWR = str
FFCLK1_IBUF_LOW_PWR__TYPE = str
FFCLK1_IBUF_LOW_PWR__RAW = str
FFCLK1_IOSTANDARD = str
FFCLK1_IOSTANDARD__TYPE = str
FFCLK1_IOSTANDARD__RAW = str
DLY_LANE0_ODELAY = int
DLY_LANE0_ODELAY__TYPE = str
DLY_LANE0_ODELAY__RAW = str
DLY_LANE0_IDELAY = int
DLY_LANE0_IDELAY__TYPE = str
DLY_LANE0_IDELAY__RAW = str
DLY_LANE1_ODELAY = int
DLY_LANE1_ODELAY__TYPE = str
DLY_LANE1_ODELAY__RAW = str
DLY_LANE1_IDELAY = int
DLY_LANE1_IDELAY__TYPE = str
DLY_LANE1_IDELAY__RAW = str
DLY_CMDA = int
DLY_CMDA__TYPE = str
DLY_CMDA__RAW = str
DLY_PHASE = int
DLY_PHASE__TYPE = str
DLY_PHASE__RAW = str
DEFAULT_STATUS_MODE = int
DEFAULT_STATUS_MODE__TYPE = str
DEFAULT_STATUS_MODE__RAW = str
LD_DLY_LANE0_ODELAY = int
LD_DLY_LANE0_ODELAY__TYPE = str
LD_DLY_LANE0_ODELAY__RAW = str
LD_DLY_LANE0_IDELAY = int
LD_DLY_LANE0_IDELAY__TYPE = str
LD_DLY_LANE0_IDELAY__RAW = str
LD_DLY_LANE1_ODELAY = int
LD_DLY_LANE1_ODELAY__TYPE = str
LD_DLY_LANE1_ODELAY__RAW = str
LD_DLY_LANE1_IDELAY = int
LD_DLY_LANE1_IDELAY__TYPE = str
LD_DLY_LANE1_IDELAY__RAW = str
LD_DLY_CMDA = int
LD_DLY_CMDA__TYPE = str
LD_DLY_CMDA__RAW = str
LD_DLY_PHASE = int
LD_DLY_PHASE__TYPE = str
LD_DLY_PHASE__RAW = str
DLY_SET = int
DLY_SET__TYPE = str
DLY_SET__RAW = str
T_RFC = int
T_RFC__TYPE = str
T_RFC__RAW = str
T_REFI = int
T_REFI__TYPE = str
T_REFI__RAW = str
DLY_DQ_IDELAY = int
DLY_DQ_IDELAY__TYPE = str
DLY_DQ_IDELAY__RAW = str
DLY_DQ_ODELAY = int
DLY_DQ_ODELAY__TYPE = str
DLY_DQ_ODELAY__RAW = str
DLY_DQS_IDELAY = int
DLY_DQS_IDELAY__TYPE = str
DLY_DQS_IDELAY__RAW = str
DLY_DQS_ODELAY = int
DLY_DQS_ODELAY__TYPE = str
DLY_DQS_ODELAY__RAW = str
DLY_DM_ODELAY = int
DLY_DM_ODELAY__TYPE = str
DLY_DM_ODELAY__RAW = str
DLY_CMDA_ODELAY = int
DLY_CMDA_ODELAY__TYPE = str
DLY_CMDA_ODELAY__RAW = str
DLY_LANE0_DQS_WLV_IDELAY = int
DLY_LANE0_DQS_WLV_IDELAY__TYPE = str
DLY_LANE0_DQS_WLV_IDELAY__RAW = str
DLY_LANE1_DQS_WLV_IDELAY = int
DLY_LANE1_DQS_WLV_IDELAY__TYPE = str
DLY_LANE1_DQS_WLV_IDELAY__RAW = str
DQSTRI_FIRST = int
DQSTRI_FIRST__TYPE = str
DQSTRI_FIRST__RAW = str
DQSTRI_LAST = int
DQSTRI_LAST__TYPE = str
DQSTRI_LAST__RAW = str
DQTRI_FIRST = int
DQTRI_FIRST__TYPE = str
DQTRI_FIRST__RAW = str
DQTRI_LAST = int
DQTRI_LAST__TYPE = str
DQTRI_LAST__RAW = str
WBUF_DLY_DFLT = int
WBUF_DLY_DFLT__TYPE = str
WBUF_DLY_DFLT__RAW = str
WBUF_DLY_WLV = int
WBUF_DLY_WLV__TYPE = str
WBUF_DLY_WLV__RAW = str
INITIALIZE_OFFSET = int
INITIALIZE_OFFSET__TYPE = str
INITIALIZE_OFFSET__RAW = str
REFRESH_OFFSET = int
REFRESH_OFFSET__TYPE = str
REFRESH_OFFSET__RAW = str
WRITELEV_OFFSET = int
WRITELEV_OFFSET__TYPE = str
WRITELEV_OFFSET__RAW = str
READ_PATTERN_OFFSET = int
READ_PATTERN_OFFSET__TYPE = str
READ_PATTERN_OFFSET__RAW = str
WRITE_BLOCK_OFFSET = int
WRITE_BLOCK_OFFSET__TYPE = str
WRITE_BLOCK_OFFSET__RAW = str
READ_BLOCK_OFFSET = int
READ_BLOCK_OFFSET__TYPE = str
READ_BLOCK_OFFSET__RAW = str
STATUS_SEQ_SHFT = int
STATUS_SEQ_SHFT__TYPE = str
STATUS_SEQ_SHFT__RAW = str
STATUS_2LSB_SHFT = int
STATUS_2LSB_SHFT__TYPE = str
STATUS_2LSB_SHFT__RAW = str
STATUS_MSB_RSHFT = int
STATUS_MSB_RSHFT__TYPE = str
STATUS_MSB_RSHFT__RAW = str
STATUS_PSHIFTER_RDY_MASK = int
STATUS_PSHIFTER_RDY_MASK__TYPE = str
STATUS_PSHIFTER_RDY_MASK__RAW = str
FRAME_START_ADDRESS = int
FRAME_START_ADDRESS__TYPE = str
FRAME_START_ADDRESS__RAW = str
FRAME_START_ADDRESS_INC = int
FRAME_START_ADDRESS_INC__TYPE = str
FRAME_START_ADDRESS_INC__RAW = str
LAST_BUF_FRAME = int
LAST_BUF_FRAME__TYPE = str
LAST_BUF_FRAME__RAW = str
CAMSYNC_DELAY = int
CAMSYNC_DELAY__TYPE = str
CAMSYNC_DELAY__RAW = str
FRAME_FULL_WIDTH = int
FRAME_FULL_WIDTH__TYPE = str
FRAME_FULL_WIDTH__RAW = str
AFI_LO_ADDR64 = int
AFI_LO_ADDR64__TYPE = str
AFI_LO_ADDR64__RAW = str
AFI_SIZE64 = int
AFI_SIZE64__TYPE = str
AFI_SIZE64__RAW = str
WINDOW_WIDTH = int
WINDOW_WIDTH__TYPE = str
WINDOW_WIDTH__RAW = str
WINDOW_HEIGHT = int
WINDOW_HEIGHT__TYPE = str
WINDOW_HEIGHT__RAW = str
WINDOW_X0 = int
WINDOW_X0__TYPE = str
WINDOW_X0__RAW = str
WINDOW_Y0 = int
WINDOW_Y0__TYPE = str
WINDOW_Y0__RAW = str
SCANLINE_STARTX = int
SCANLINE_STARTX__TYPE = str
SCANLINE_STARTX__RAW = str
SCANLINE_STARTY = int
SCANLINE_STARTY__TYPE = str
SCANLINE_STARTY__RAW = str
SCANLINE_EXTRA_PAGES = int
SCANLINE_EXTRA_PAGES__TYPE = str
SCANLINE_EXTRA_PAGES__RAW = str
TILED_STARTX = int
TILED_STARTX__TYPE = str
TILED_STARTX__RAW = str
TILED_STARTY = int
TILED_STARTY__TYPE = str
TILED_STARTY__RAW = str
TILED_EXTRA_PAGES = int
TILED_EXTRA_PAGES__TYPE = str
TILED_EXTRA_PAGES__RAW = str
TILED_KEEP_OPEN = int
TILED_KEEP_OPEN__TYPE = str
TILED_KEEP_OPEN__RAW = str
TILE_WIDTH = int
TILE_WIDTH__TYPE = str
TILE_WIDTH__RAW = str
TILE_HEIGHT = int
TILE_HEIGHT__TYPE = str
TILE_HEIGHT__RAW = str
TILE_VSTEP = int
TILE_VSTEP__TYPE = str
TILE_VSTEP__RAW = str
TEST01_START_FRAME = int
TEST01_START_FRAME__TYPE = str
TEST01_START_FRAME__RAW = str
TEST01_NEXT_PAGE = int
TEST01_NEXT_PAGE__TYPE = str
TEST01_NEXT_PAGE__RAW = str
TEST01_SUSPEND = int
TEST01_SUSPEND__TYPE = str
TEST01_SUSPEND__RAW = str
TEST_INITIAL_BURST = int
TEST_INITIAL_BURST__TYPE = str
TEST_INITIAL_BURST__RAW = str
NUM_INTERRUPTS = int
NUM_INTERRUPTS__TYPE = str
NUM_INTERRUPTS__RAW = str
AXI_RDADDR_LATENCY = int
AXI_RDADDR_LATENCY__TYPE = str
AXI_RDADDR_LATENCY__RAW = str
AXI_WRADDR_LATENCY = int
AXI_WRADDR_LATENCY__TYPE = str
AXI_WRADDR_LATENCY__RAW = str
AXI_WRDATA_LATENCY = int
AXI_WRDATA_LATENCY__TYPE = str
AXI_WRDATA_LATENCY__RAW = str
AXI_TASK_HOLD = float
AXI_TASK_HOLD__TYPE = str
AXI_TASK_HOLD__RAW = str
SIMUL_AXI_READ_WIDTH = int
SIMUL_AXI_READ_WIDTH__TYPE = str
SIMUL_AXI_READ_WIDTH__RAW = str
MEMCLK_PERIOD = float
MEMCLK_PERIOD__TYPE = str
MEMCLK_PERIOD__RAW = str
FCLK0_PERIOD = float
FCLK0_PERIOD__TYPE = str
FCLK0_PERIOD__RAW = str
FCLK1_PERIOD = float
FCLK1_PERIOD__TYPE = str
FCLK1_PERIOD__RAW = str
LWIR_DATA_FILE1 = str
LWIR_DATA_FILE1__TYPE = str
LWIR_DATA_FILE1__RAW = str
LWIR_DATA_FILE2 = str
LWIR_DATA_FILE2__TYPE = str
LWIR_DATA_FILE2__RAW = str
LWIR_DATA_FILE3 = str
LWIR_DATA_FILE3__TYPE = str
LWIR_DATA_FILE3__RAW = str
LWIR_DATA_FILE4 = str
LWIR_DATA_FILE4__TYPE = str
LWIR_DATA_FILE4__RAW = str
LWIR_WINDOW_WIDTH = int
LWIR_WINDOW_WIDTH__TYPE = str
LWIR_WINDOW_WIDTH__RAW = str
LWIR_WINDOW_HEIGHT = int
LWIR_WINDOW_HEIGHT__TYPE = str
LWIR_WINDOW_HEIGHT__RAW = str
LWIR_TELEMETRY = int
LWIR_TELEMETRY__TYPE = str
LWIR_TELEMETRY__RAW = str
LWIR_FRAME_PERIOD = int
LWIR_FRAME_PERIOD__TYPE = str
LWIR_FRAME_PERIOD__RAW = str
LWIR_FRAME_DELAY = int
LWIR_FRAME_DELAY__TYPE = str
LWIR_FRAME_DELAY__RAW = str
LWIR_MS_PERIOD = int
LWIR_MS_PERIOD__TYPE = str
LWIR_MS_PERIOD__RAW = str
LWIR_TELEMETRY_REV = int
LWIR_TELEMETRY_REV__TYPE = str
LWIR_TELEMETRY_REV__RAW = str
LWIR_TELEMETRY_STATUS = int
LWIR_TELEMETRY_STATUS__TYPE = str
LWIR_TELEMETRY_STATUS__RAW = str
LWIR_TELEMETRY_SREV = int
LWIR_TELEMETRY_SREV__TYPE = str
LWIR_TELEMETRY_SREV__RAW = str
LWIR_TELEMETRY_TEMP_COUTS = int
LWIR_TELEMETRY_TEMP_COUTS__TYPE = str
LWIR_TELEMETRY_TEMP_COUTS__RAW = str
LWIR_TELEMETRY_TEMP_KELVIN = int
LWIR_TELEMETRY_TEMP_KELVIN__TYPE = str
LWIR_TELEMETRY_TEMP_KELVIN__RAW = str
LWIR_TELEMETRY_TEMP_LAST_KELVIN = int
LWIR_TELEMETRY_TEMP_LAST_KELVIN__TYPE = str
LWIR_TELEMETRY_TEMP_LAST_KELVIN__RAW = str
LWIR_TELEMETRY_TIME_LAST_MS = int
LWIR_TELEMETRY_TIME_LAST_MS__TYPE = str
LWIR_TELEMETRY_TIME_LAST_MS__RAW = str
LWIR_TELEMETRY_AGC_ROI_TOP = int
LWIR_TELEMETRY_AGC_ROI_TOP__TYPE = str
LWIR_TELEMETRY_AGC_ROI_TOP__RAW = str
LWIR_TELEMETRY_AGC_ROI_LEFT = int
LWIR_TELEMETRY_AGC_ROI_LEFT__TYPE = str
LWIR_TELEMETRY_AGC_ROI_LEFT__RAW = str
LWIR_TELEMETRY_AGC_ROI_BOTTOM = int
LWIR_TELEMETRY_AGC_ROI_BOTTOM__TYPE = str
LWIR_TELEMETRY_AGC_ROI_BOTTOM__RAW = str
LWIR_TELEMETRY_AGC_ROI_RIGHT = int
LWIR_TELEMETRY_AGC_ROI_RIGHT__TYPE = str
LWIR_TELEMETRY_AGC_ROI_RIGHT__RAW = str
LWIR_TELEMETRY_AGC_HIGH = int
LWIR_TELEMETRY_AGC_HIGH__TYPE = str
LWIR_TELEMETRY_AGC_HIGH__RAW = str
LWIR_TELEMETRY_AGC_LOW = int
LWIR_TELEMETRY_AGC_LOW__TYPE = str
LWIR_TELEMETRY_AGC_LOW__RAW = str
LWIR_TELEMETRY_VIDEO_FORMAT = int
LWIR_TELEMETRY_VIDEO_FORMAT__TYPE = str
LWIR_TELEMETRY_VIDEO_FORMAT__RAW = str
LWIR_GPIO_IN = int
LWIR_GPIO_IN__TYPE = str
LWIR_GPIO_IN__RAW = str
SENSOR_IMAGE_TYPE0 = str
SENSOR_IMAGE_TYPE0__TYPE = str
SENSOR_IMAGE_TYPE0__RAW = str
SENSOR_IMAGE_TYPE1 = str
SENSOR_IMAGE_TYPE1__TYPE = str
SENSOR_IMAGE_TYPE1__RAW = str
SENSOR_IMAGE_TYPE2 = str
SENSOR_IMAGE_TYPE2__TYPE = str
SENSOR_IMAGE_TYPE2__RAW = str
SENSOR_IMAGE_TYPE3 = str
SENSOR_IMAGE_TYPE3__TYPE = str
SENSOR_IMAGE_TYPE3__RAW = str
SIMULATE_CMPRS_CMODE0 = int
SIMULATE_CMPRS_CMODE0__TYPE = str
SIMULATE_CMPRS_CMODE0__RAW = str
SIMULATE_CMPRS_CMODE1 = int
SIMULATE_CMPRS_CMODE1__TYPE = str
SIMULATE_CMPRS_CMODE1__RAW = str
SIMULATE_CMPRS_CMODE2 = int
SIMULATE_CMPRS_CMODE2__TYPE = str
SIMULATE_CMPRS_CMODE2__RAW = str
SIMULATE_CMPRS_CMODE3 = int
SIMULATE_CMPRS_CMODE3__TYPE = str
SIMULATE_CMPRS_CMODE3__RAW = str
SENSOR12BITS_NGPL = int
SENSOR12BITS_NGPL__TYPE = str
SENSOR12BITS_NGPL__RAW = str
SENSOR12BITS_NVLO = int
SENSOR12BITS_NVLO__TYPE = str
SENSOR12BITS_NVLO__RAW = str
SENSOR12BITS_TMD = int
SENSOR12BITS_TMD__TYPE = str
SENSOR12BITS_TMD__RAW = str
SENSOR12BITS_TDDO = int
SENSOR12BITS_TDDO__TYPE = str
SENSOR12BITS_TDDO__RAW = str
SENSOR12BITS_TDDO1 = int
SENSOR12BITS_TDDO1__TYPE = str
SENSOR12BITS_TDDO1__RAW = str
HISTOGRAM_LEFT = int
HISTOGRAM_LEFT__TYPE = str
HISTOGRAM_LEFT__RAW = str
HISTOGRAM_TOP = int
HISTOGRAM_TOP__TYPE = str
HISTOGRAM_TOP__RAW = str
HISTOGRAM_WIDTH = int
HISTOGRAM_WIDTH__TYPE = str
HISTOGRAM_WIDTH__RAW = str
HISTOGRAM_HEIGHT = int
HISTOGRAM_HEIGHT__TYPE = str
HISTOGRAM_HEIGHT__RAW = str
HISTOGRAM_START_PAGE = int
HISTOGRAM_START_PAGE__TYPE = str
HISTOGRAM_START_PAGE__RAW = str
FRAME_WIDTH_ROUND_BITS = int
FRAME_WIDTH_ROUND_BITS__TYPE = str
FRAME_WIDTH_ROUND_BITS__RAW = str
WOI_WIDTH = int
WOI_WIDTH__TYPE = str
WOI_WIDTH__RAW = str
WOI_HEIGHT = int
WOI_HEIGHT__TYPE = str
WOI_HEIGHT__RAW = str
QUADRANTS_PXD_HACT_VACT = int
QUADRANTS_PXD_HACT_VACT__TYPE = str
QUADRANTS_PXD_HACT_VACT__RAW = str
SENSOR_PRIORITY = int
SENSOR_PRIORITY__TYPE = str
SENSOR_PRIORITY__RAW = str
